﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Net;
using System.Xml;

namespace MusicSearcher
{
    public partial class Form1 : Form
    {
        // Search type
        private bool _searchArtist = true;

        public Form1()
        {
            InitializeComponent();
        }

        private void cmdSearch_Click(object sender, EventArgs e)
        {
            // If we have no criteria, don't search...
            if (String.IsNullOrEmpty(tbPhrase.Text))
                return;

            // We have criteria, so clear UI elements
            lbResults.SelectedIndexChanged -= new EventHandler(lbResults_SelectedIndexChanged);
            lbResults.Items.Clear();
            lbMore.Items.Clear();
            cmdMore.Enabled = false;
            lblMore.Enabled = false;

            // Maintain the artist/title setting so we don't have logical
            // errors when asking for more information (user could change
            // the button setting before clicking "More" and goof up the
            // search state).
            _searchArtist = rbArtist.Checked;

            // Pull the search criteria. Since we're making a Web search, spaces
            // in the search phrase must be replaced with plus signs.
            string criteria = tbPhrase.Text.Replace(' ', '+');
            string url = _searchArtist ?
                String.Format("http://musicbrainz.org/ws/1/artist/?type=xml&name={0}", criteria) :
                String.Format("http://musicbrainz.org/ws/1/track/?type=xml&title={0}", criteria);

            // Request the information. Note the only representation
            // available to us for this service is XML, so we'll load
            // the response into an XML document for scoring.
            XmlDocument xdoc = null;
            try
            {
                // Request results XML
                xdoc = CallService(url);
            }
            catch (WebException ex)
            {
                string errorMessage = "Error requesting information from service.";
                if (ex.Status == WebExceptionStatus.ProtocolError)
                {
                    // Report REST error to user. The error status codes are prescribed
                    // by the service, see http://wiki.musicbrainz.org/XMLWebService.
                    errorMessage = ParseWebError(((HttpWebResponse)ex.Response).StatusCode);
                }

                // Display error
                lbResults.Items.Add(errorMessage);
                return;
            }
            catch
            {
                // Some error...
                lbResults.Items.Add("Error requesting information from service.");
                return;
            }

            // Pull the results based on score
            XmlNodeList itemNodes = null;
            try
            {
                // Since the attribute we're filtering has a namespace,
                // we need to add it to the namespace manager.
                XmlNamespaceManager mgr = new XmlNamespaceManager(xdoc.NameTable);
                mgr.AddNamespace("a", "http://musicbrainz.org/ns/mmd-1.0#");
                mgr.AddNamespace("ext", "http://musicbrainz.org/ns/ext-1.0#");

                // Create the XPath expression. The returned XML differs for
                // an artist or track search...
                string xpath = _searchArtist ?
                    String.Format("/a:metadata/a:artist-list/a:artist[./@ext:score >= '{0}']", nudScore.Value.ToString()) :
                    String.Format("/a:metadata/a:track-list/a:track[./@ext:score >= '{0}']", nudScore.Value.ToString());

                // XPath the nodes from the results document
                itemNodes = xdoc.SelectNodes(xpath, mgr);

                // Check for returned data (filtered by score)
                if (itemNodes.Count > 0)
                {
                    // We have data... The XML differs between an artist and
                    // track search, so we'll cover each case separately.
                    if (_searchArtist)
                    {
                        // Artist search
                        foreach (XmlNode artistNode in itemNodes)
                        {
                            // The artist name is the first child of the
                            // artist node (the node we're working with)
                            MusicListItem li = new MusicListItem(artistNode.FirstChild.InnerText, artistNode.Attributes["id"].Value);
                            lbResults.Items.Add(li);
                        }
                    }
                    else
                    {
                        // Track search
                        foreach (XmlNode trackNode in itemNodes)
                        {
                            // The track title is the first child of the track
                            // node (the node we're working with). The track may
                            // be featured on several albums (hence more than
                            // one track node), so we'll build the display
                            // value based on track title and album name. The
                            // album name is a bit harder to dig out so we'll
                            // XPath it. We'll take the first one we find since
                            // the service doesn't appear to roll multiple
                            // albums into this collection but rather reports
                            // each instance of the title as a separate title
                            // element...
                            XmlNode albumNode = trackNode.SelectSingleNode("./a:release-list/a:release/a:title", mgr);
                            XmlNode artistNode = trackNode.SelectSingleNode("./a:artist/a:name", mgr);
                            string displayValue = String.Format("{0} (Artist: '{1}', Album: '{2}') ", trackNode.FirstChild.InnerText, artistNode.InnerText, albumNode.InnerText);
                            MusicListItem li = new MusicListItem(displayValue, trackNode.Attributes["id"].Value);
                            lbResults.Items.Add(li);
                        }
                    }

                    // We have data, so enable the listbox selection
                    lbResults.SelectedIndexChanged += new EventHandler(lbResults_SelectedIndexChanged);
                }
                else
                {
                    // No data...
                    lbResults.Items.Add("No items matched your search criteria.");
                }
            }
            catch
            {
                // Some error...
                lbResults.Items.Add("Error interpreting information from service.");
            }
        }

        private void lbResults_SelectedIndexChanged(object sender, EventArgs e)
        {
            // We have items, so enable the more information button...
            cmdMore.Enabled = true;
        }

        private void cmdMore_Click(object sender, EventArgs e)
        {
            // Pull the selected result item
            MusicListItem li = lbResults.SelectedItem as MusicListItem;
            if (li == null)
                return;

            // The search results listbox has the artist/title ID we'll
            // need. If the artist, we'll pull the band members. If a
            // track we'll pull the duration (we had this from the track
            // information we already obtained, but we'll re-retrive
            // it to demonstrate using the service).
            string url = _searchArtist ?
                String.Format("http://musicbrainz.org/ws/1/artist/{0}?type=xml&inc=artist-rels", li.ID) :
                String.Format("http://musicbrainz.org/ws/1/track/{0}?type=xml", li.ID);

            // Request the information. Note the only representation
            // available to us for this service is (again) XML.
            XmlDocument xdoc = null;
            try
            {
                // Request results XML
                xdoc = CallService(url);
            }
            catch (WebException ex)
            {
                string errorMessage = "Error requesting information from service.";
                if (ex.Status == WebExceptionStatus.ProtocolError)
                {
                    // Report REST error to user. The error status codes are prescribed
                    // by the service, see http://wiki.musicbrainz.org/XMLWebService.
                    errorMessage = ParseWebError(((HttpWebResponse)ex.Response).StatusCode);
                }

                // Display error
                lbMore.Items.Add(errorMessage);
                return;
            }
            catch
            {
                // Some error...
                lbMore.Items.Add("Error requesting information from service.");
                return;
            }

            // Pull the results based on score
            XmlNodeList itemNodes = null;
            try
            {
                // Since the attribute we're filtering has a namespace,
                // we need to add it to the namespace manager.
                XmlNamespaceManager mgr = new XmlNamespaceManager(xdoc.NameTable);
                mgr.AddNamespace("a", "http://musicbrainz.org/ns/mmd-1.0#");
                mgr.AddNamespace("ext", "http://musicbrainz.org/ns/ext-1.0#");

                // Create the XPath expression. The returned XML differs for
                // an artist or track search...
                string xpath = _searchArtist ?
                    "/a:metadata/a:artist" :
                    "/a:metadata/a:track";

                // XPath the nodes from the results document
                itemNodes = xdoc.SelectNodes(xpath, mgr);

                // Check for returned data (filtered by score)
                if (itemNodes.Count > 0)
                {
                    // We have data... The XML differs between an artist and
                    // track search, so we'll cover each case separately.
                    if (_searchArtist)
                    {
                        // Artist search...note the artist could be a group
                        // or a person. If a group we'll show the band
                        // members. In both cases we'll show their
                        // birthdate (now I feel really old).
                        XmlNode artistNode = itemNodes[0];
                        if (artistNode.Attributes["type"].Value == "Group")
                        {
                            // Pull the group name
                            XmlNode nameNode = artistNode.SelectSingleNode("./a:name", mgr);
                            lbMore.Items.Add(String.Format("Band name: {0}", nameNode.InnerText));

                            // There will be a collection of person
                            XmlNodeList memberNodes = artistNode.SelectNodes("./a:relation-list/a:relation", mgr);
                            foreach (XmlNode memberNode in memberNodes)
                            {
                                nameNode = memberNode.FirstChild.SelectSingleNode("./a:name", mgr);
                                XmlNode dateNode = memberNode.FirstChild.SelectSingleNode("./a:life-span", mgr);
                                DateTime birthdate = dateNode != null ? DateTime.Parse(dateNode.Attributes["begin"].Value) : DateTime.MaxValue;
                                lbMore.Items.Add(String.Format("\t{0}, born {1}", nameNode.InnerText, birthdate < DateTime.MaxValue ? birthdate.ToString("MMMM d, yyyy") : "(unknown)"));
                            }
                        }
                        else if (artistNode.Attributes["type"].Value == "Person")
                        {
                            XmlNode nameNode = artistNode.SelectSingleNode("./a:name", mgr);
                            XmlNode dateNode = artistNode.SelectSingleNode("./a:life-span", mgr);
                            DateTime birthdate = dateNode != null ? DateTime.Parse(dateNode.Attributes["begin"].Value) : DateTime.MaxValue;
                            lbMore.Items.Add(String.Format("Artist: {0}, born {1}", nameNode.InnerText, birthdate < DateTime.MaxValue ? birthdate.ToString("MMMM d, yyyy") : "(unknown)"));
                        }
                    }
                    else
                    {
                        // Track search
                        XmlNode trackNode = itemNodes[0];

                        // All we'll pull here is the duration. The duration is exposed
                        // in milliseconds, so adding "0000" to the end represents
                        // hundreds of nanoseconds, which is perfect for TimeSpan.
                        // TimeSpan can then do the math for us.
                        XmlNode durationNode = trackNode.SelectSingleNode("./a:duration", mgr);
                        Int64 durationValue = 0;
                        Int64.TryParse(durationNode.InnerText + "0000", out durationValue);
                        TimeSpan duration = new TimeSpan(durationValue); // expressed as "ticks"
                        string displayValue = String.Format("Song duration: {0}:{1}", duration.Minutes, duration.Seconds.ToString("00"));
                        lbMore.Items.Add(displayValue);
                    }

                    // Enable controls
                    lblMore.Enabled = true;
                    lbMore.Enabled = true;
                }
                else
                {
                    // No data...
                    lbMore.Items.Add("No items matched your search criteria.");
                }
            }
            catch
            {
                // Some error...
                lbMore.Items.Add("Error interpreting information from service.");
            }
        }

        private XmlDocument CallService(string url)
        {
            HttpWebRequest request = (HttpWebRequest)WebRequest.Create(url);
            WebResponse response = request.GetResponse();
            XmlDocument xdoc = new XmlDocument();
            xdoc.Load(response.GetResponseStream());
            return xdoc;
        }

        private string ParseWebError(HttpStatusCode httpCode)
        {
            string errorMessage = "Error requesting information from service.";
            switch (httpCode)
            {
                case HttpStatusCode.BadRequest: // 400
                    errorMessage = "Invalid parameter value. (Programmer error.)";
                    break;

                case HttpStatusCode.Unauthorized: // 401
                    errorMessage = "Authentication required. (Programmer error.)";
                    break;

                case HttpStatusCode.NotFound: // 404
                    errorMessage = "Resource not found. There is no resource having this ID. (Programmer error.)";
                    break;
            }

            return errorMessage;
        }
    }
}
