﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Text.RegularExpressions;
using System.Xml;

using WeatherGetter.WeatherService;

namespace WeatherGetter
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        public Window1()
        {
            InitializeComponent();
        }

        private void btnGet_Click(object sender, RoutedEventArgs e)
        {
            // Validate the incoming text
            if (!String.IsNullOrEmpty(tbZip.Text) && Regex.IsMatch(tbZip.Text, @"^\d{5}([\-]\d{4})?$"))
            {
                ndfdXMLPortTypeClient ws = null;
                string latlonXml = String.Empty;
                try
                {
                    // ZIP code validated, so call Web Service with the ZIP code
                    // to determine lat/lon values.
                    ws = new ndfdXMLPortTypeClient();
                    latlonXml = ws.LatLonListZipCode(tbZip.Text);
                }
                catch
                {
                    // Some error...
                    txResults.Text = "Error contacting weather service.\r\nPlease check the ZIP code.";
                    return;
                }

                // The results will come in an XML form, like so (ZIP: 90210):
                //
                // <?xml version='1.0' ?>
                // <dwml version='1.0' xmlns:xsd="http://www.w3.org/2001/XMLSchema"
                //   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
                //   xsi:noNamespaceSchemaLocation="http://www.nws.noaa.gov/mdl/survey/pgb_survey/dev/DWMLgen/schema/DWML.xsd">
                //    <latLonList>34.0995,-118.414</latLonList>
                // </dwml>
                // 
                // Given this we need to parse out the lat/lon list. Since this XML is
                // so simple, we'll just dig it out with the DOM.
                string[] latlon = null;
                try
                {
                    XmlDocument xdoc = new XmlDocument();
                    xdoc.LoadXml(latlonXml);
                    XmlNode latlonNode = xdoc.DocumentElement.FirstChild;
                    latlon = latlonNode.InnerText.Split(',');
                }
                catch
                {
                    // Some error...
                    txResults.Text = "Error interpreting weather service response.";
                    return;
                }

                // Create the weather items of interest.
                weatherParametersType parms = new weatherParametersType();
                parms.maxt = true; // maximum temperature
                parms.mint = true; // minimum temperature
                parms.sky = true; // cloud cover, in percent

                string weatherXml = String.Empty;
                try
                {
                    // With the lat/lon in hand, we can call for weather.
                    decimal latitude = 38.99m; // Weather headquarters coordinates
                    Decimal.TryParse(latlon[0], out latitude);
                    decimal longitude = -77.01m;
                    Decimal.TryParse(latlon[1], out longitude);
                    weatherXml = ws.NDFDgen(latitude, longitude, productType.glance, DateTime.Now, DateTime.Now.AddDays(1.0), parms);
                }
                catch
                {
                    // Some error...
                    txResults.Text = "Error requesting weather forecast.";
                    return;
                }

                // The weather XML vocabulary is complex, but we'll simplify
                // by only looking at the forecasted values closest in time
                // (the first child node of the given weather condition node).
                StringBuilder results = new StringBuilder();
                try
                {
                    XmlDocument xdoc = new XmlDocument();
                    xdoc.LoadXml(weatherXml);
                    XmlNode parmsNode = xdoc.SelectSingleNode("/dwml/data/parameters");

                    // The <parameters> node contains the interesting weather
                    // information. First we'll pull the temperatures.
                    XmlNodeList tempNodes = parmsNode.SelectNodes("./temperature");
                    foreach (XmlNode node in tempNodes)
                    {
                        // The temperature node will have an attribute that
                        // indicates max/min temperature.
                        if (node.Attributes["type"].Value == "maximum")
                        {
                            // Found max temp...
                            results.AppendFormat("Maximum temperature: {0}°F\r\n", node.ChildNodes[1].InnerText);
                        }
                        else
                        {
                            // Found max temp...
                            results.AppendFormat("Minumum temperature: {0}°F\r\n", node.ChildNodes[1].InnerText);
                        }
                    }

                    // Next we'll pull the cloud cover. Note the second child node
                    // contains the information we seek.
                    XmlNode cloudNode = parmsNode.SelectSingleNode("./cloud-amount");
                    results.AppendFormat("Cloud coverage: {0}%\r\n", cloudNode.ChildNodes[1].InnerText);

                    // Next we find the current weather conditions. We'll take the first
                    // condition we find.
                    XmlNode conditionNode = parmsNode.SelectSingleNode("./weather/weather-conditions");

                    // Given the condition, we'll take the first value we find
                    if (conditionNode.HasChildNodes)
                    {
                        XmlNode valueNode = conditionNode.FirstChild;

                        // Pull the information
                        if (valueNode.Attributes.Count > 0)
                        {
                            results.AppendFormat("Forecast: {0}{1} {2}\r\n",
                                valueNode.Attributes["coverage"].Value,
                                valueNode.Attributes["intensity"].Value != "none" ?
                                    (" of " + valueNode.Attributes["intensity"].Value + " ") :
                                    String.Empty,
                                valueNode.Attributes["weather-type"].Value);
                        }
                    }
                    else
                    {
                        // No forecast to show...
                        results.Append("Forecast: unavailable at this time.");
                    }

                    // Pull the weather icon. The icon we'll use is the second
                    // child node.
                    XmlNode iconNode = parmsNode.SelectSingleNode("conditions-icon");
                    icon.Source = new Uri(iconNode.ChildNodes[1].InnerText);
                }
                catch
                {
                    // Some error...
                    txResults.Text = "Error interpreting weather service response.";
                    return;
                }

                // Show forecast results
                txResults.Text = results.ToString();
            }
            else
            {
                // Invalid ZIP code
                txResults.Text = "Please provide a valid US ZIP code.";
            }
        }
    }
}
