﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Web;
using System.IO;

namespace HttpExerciser
{
    public class PacketProcessor
    {
        #region Constants

        private const string HtmlFormat = "<html>\r\n<head>\r\n<title>My Response</title>\r\n</head>\r\n<body>\r\n<p>{0}</p><p>See also: <a href=\"http://tempuri.org/MoreInfo/\">http://tempuri.org/MoreInfo/</a></p></body>\r\n</html>";
        private const string XmlFormat = "<?xml version=\"1.0\"?><myResponse><data>{0}</data><seeAlso>http://tempuri.org/MoreInfo/</seeAlso></myResponse>";
        private const string AtomFormat = "<?xml version=\"1.0\"?><feed xmlns=\"http://www.w3.org/2005/Atom\"><title>.NET REST Services</title><link href=\"http://tempuri.org/MoreInfo/\" rel=\"self\"/><updated>2008-08-08T08:08:08Z</updated><author><name>Kenn Scribner</name><email>kenn@tempuri.org</email></author><id>urn:uuid:A8B6A480-B1F8-459e-9A41-51E85719A359</id><entry><title>My Response</title><link href=\"http://tempuri.org/Atom\"/><id>urn:uuid:47D73759-7B71-46b2-854A-4ED7637B3DEC</id><updated>2008-08-08T08:08:08Z</updated><summary>{0}</summary></entry></feed>";
        private const string JsonFormat = "{{'myResponse': '{0}', 'seeAlso': 'http://tempuri.org/MoreInfo/'}}";
        private const string ErrorFormat = "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01//EN\"\"http://www.w3.org/TR/html4/strict.dtd\"><HTML><HEAD><TITLE>{0}</TITLE><META HTTP-EQUIV=\"Content-Type\" Content=\"text/html; charset=us-ascii\"></HEAD><BODY><h2>{0}</h2><hr><p>{1}</p></BODY></HTML>";

        #endregion

        #region ProcessRequest

        public RequestPacketInfo ProcessRequest(HttpListenerRequest request)
        {
            // Record the request information
            RequestPacketInfo requestPacket = new RequestPacketInfo(PacketInfo.PacketTypes.Request, (Int32)request.ContentLength64, request.RawUrl);

            // Build method header
            List<string> headerList = new List<string>();
            string methodHeader = String.Format("{0} {1} HTTP/{2}", request.HttpMethod.ToUpper(), request.Url.PathAndQuery, request.ProtocolVersion);
            headerList.Add(methodHeader);

            // Retrieve the request packet headers
            for (Int32 i = request.Headers.Count - 1; i >= 0; i--)
            {
                headerList.Add(String.Format("{0}: {1}", request.Headers.GetKey(i), request.Headers[i]));
            }
            requestPacket.PacketHeaders = headerList;

            // Retrieve the request packet HTTP verb
            requestPacket.PacketVerb = RequestPacketInfo.ParseVerb(request.HttpMethod);

            // Retrieve the desired content type
            if (Properties.Settings.Default.ContentNegotiation)
            {
                // Retrieve the desired content type from the Accept header
                requestPacket.PacketContentType = RequestPacketInfo.ParseContentType(request.Headers["Accept"]);
            }
            else
            {
                // Decode the incoming URI
                string decodedUrl = HttpUtility.UrlDecode(request.RawUrl);
                requestPacket.PacketContentType = RequestPacketInfo.ParseContentUrl(decodedUrl);
            }

            // Retrieve the request packet input stream (may be empty)
            using (StreamReader rdr = new StreamReader(request.InputStream))
            {
                try
                {
                    string encodedContents = rdr.ReadToEnd();
                    string[] contents = encodedContents.Split('=');
                    requestPacket.PacketContents = contents[1];
                }
                catch
                {
                    // Some error...will just assign empty
                    requestPacket.PacketContents = String.Empty;
                }
            }

            return requestPacket;
        }

        #endregion

        #region ProcessResponse

        public ResponsePacketInfo ProcessResponse(RequestPacketInfo requestPacket)
        {
            // Check that we're allowed to process the given resource
            ResponsePacketInfo responsePacket = new ResponsePacketInfo(PacketInfo.PacketTypes.Response, 0, requestPacket.PacketUri);
            if (!requestPacket.IsProperContentType)
            {
                // No, so return a 404
                responsePacket.PacketContentType = PacketInfo.PacketContentTypes.Html;
                responsePacket.PacketContents = FormatError(ResponsePacketInfo.ResponseCodes.Http404);
                responsePacket.AssignResponse(ResponsePacketInfo.ResponseCodes.Http404);
            }
            else if (!requestPacket.IsProperVerb)
            {
                // Not a verb we're willing to support...return a 405
                responsePacket.PacketContentType = PacketInfo.PacketContentTypes.Html;
                responsePacket.PacketContents = FormatError(ResponsePacketInfo.ResponseCodes.Http405);
                responsePacket.AssignResponse(ResponsePacketInfo.ResponseCodes.Http405);
            }
            else
            {
                // Create response packet
                string responseContent = String.Empty;
                ResponsePacketInfo.ResponseCodes code = ResponsePacketInfo.ResponseCodes.Http500;
                switch (requestPacket.PacketVerb)
                {
                    case RequestPacketInfo.PacketVerbs.HEAD:
                        responsePacket.PacketContentType = requestPacket.PacketContentType;
                        responsePacket.IsHeadResponse = true;
                        responsePacket.AssignResponse(ResponsePacketInfo.ResponseCodes.Http200);
                        break;

                    case RequestPacketInfo.PacketVerbs.GET:
                        responseContent = ProcessGetVerb(requestPacket);
                        responsePacket.PacketContentType = requestPacket.PacketContentType;
                        responsePacket.AssignResponse(!String.IsNullOrEmpty(responseContent) ?
                            ResponsePacketInfo.ResponseCodes.Http200 :
                            ResponsePacketInfo.ResponseCodes.Http204);
                        break;

                    case RequestPacketInfo.PacketVerbs.PUT:
                        code = ProcessPutVerb(requestPacket);
                        responsePacket.AssignResponse(code);
                        if (code == ResponsePacketInfo.ResponseCodes.Http409)
                        {
                            // Provide guidance
                            responsePacket.PacketContentType = PacketInfo.PacketContentTypes.Html;
                            responseContent = FormatError(ResponsePacketInfo.ResponseCodes.Http409);
                        }
                        else
                        {
                            responsePacket.PacketContentType = PacketInfo.PacketContentTypes.None;
                        }
                        break;

                    case RequestPacketInfo.PacketVerbs.POST:
                        code = ProcessPostVerb(requestPacket);
                        responsePacket.AssignResponse(code);
                        break;

                    case RequestPacketInfo.PacketVerbs.DELETE:
                        code = ProcessDeleteVerb(requestPacket);
                        responsePacket.AssignResponse(code);
                        responsePacket.PacketContentType = PacketInfo.PacketContentTypes.None;
                        break;

                    default:
                        break;
                }
                byte[] buffer = System.Text.Encoding.UTF8.GetBytes(responseContent);
                responsePacket.PacketSize = buffer.Length;
                responsePacket.PacketContents = responseContent;
            }

            return responsePacket;
        }

        public ResponsePacketInfo ProcessResponse(RequestPacketInfo requestPacket, ResponsePacketInfo.ResponseCodes code)
        {
            // Create the response packet
            ResponsePacketInfo responsePacket = new ResponsePacketInfo(PacketInfo.PacketTypes.Response, 0, requestPacket.PacketUri);

            // Return appropriate response
            responsePacket.PacketContentType = PacketInfo.PacketContentTypes.Html;
            responsePacket.PacketContents = FormatError(code);
            responsePacket.AssignResponse(code);

            return responsePacket;
        }

        #endregion

        #region IssueResponse

        public void IssueResponse(HttpListenerResponse response, ResponsePacketInfo responsePacket)
        {
            byte[] buffer = System.Text.Encoding.UTF8.GetBytes(responsePacket.PacketContents);
            if (!responsePacket.IsHeadResponse)
            {
                // Establish default content type
                response.ContentLength64 = buffer.Length;
                if (responsePacket.PacketContentType != PacketInfo.PacketContentTypes.None)
                {
                    response.ContentType = responsePacket.PacketContentTypeText;
                }
            }
            else
            {
                // Return what would be the size of the entity-body
                try
                {
                    // Note if you're not using .NET 3.5 SP1, this will throw an exception
                    // stating that the Content-Length header is read-only. Clearly this
                    // is a bad thing for REST, and happily .NET 3.5 SP1 corrected things.
                    response.Headers["Content-Length"] = responsePacket.PacketSize.ToString();
                }
                catch (Exception ex)
                {
                    throw new Exception("For HEAD to be properly issued to the client you must use at least .NET 3.5 SP1 (HttpListener header behavior was updated).", ex);
                }
            }

            // Send response to client
            using (System.IO.Stream output = response.OutputStream)
            {
                output.Write(buffer, 0, buffer.Length);
                output.Close();
            }

            // Assign the return status codes
            response.StatusCode = responsePacket.PacketResponseCode;
            response.StatusDescription = responsePacket.PacketResponseCodeText;

            // Build response header
            List<string> headerList = new List<string>();
            string responseHeader = String.Format("HTTP/{0} {1} {2}", response.ProtocolVersion, (Int32)response.StatusCode, response.StatusDescription);
            headerList.Add(responseHeader);

            // Retrieve the response packet headers
            for (Int32 i = response.Headers.Count - 1; i >= 0; i--)
            {
                headerList.Add(String.Format("{0}: {1}", response.Headers.GetKey(i), response.Headers[i]));
            }
            responsePacket.PacketHeaders = headerList;
        }

        #endregion

        #region Helpers

        private string ProcessGetVerb(RequestPacketInfo requestPacket)
        {
            // If we have a resource...
            string resourceText = String.Empty;
            if (!String.IsNullOrEmpty(Properties.Settings.Default.ResourceText))
            {
                // Return the content in the desired form
                string formatString = String.Empty;
                switch (requestPacket.PacketContentType)
                {
                    case RequestPacketInfo.PacketContentTypes.Html:
                        formatString = HtmlFormat;
                        break;

                    case RequestPacketInfo.PacketContentTypes.Xml:
                        formatString = XmlFormat;
                        break;

                    case RequestPacketInfo.PacketContentTypes.Atom:
                    default:
                        formatString = AtomFormat;
                        break;

                    case RequestPacketInfo.PacketContentTypes.Json:
                        formatString = JsonFormat;
                        break;
                }

                resourceText = String.Format(formatString, Properties.Settings.Default.ResourceText);
            }
            else
            {
                // We have no data (probably through a DELETE), so return empty
                ;
            }

            return resourceText;
        }

        private ResponsePacketInfo.ResponseCodes ProcessPutVerb(RequestPacketInfo requestPacket)
        {
            // Normally we'd process the representation, extract what we needed
            // to store the created resource, and move on. To simplify things,
            // we'll accept a string in the entity body and leave the individual
            // parsing exercises for another day.
            ResponsePacketInfo.ResponseCodes code = ResponsePacketInfo.ResponseCodes.Http204;
            switch (requestPacket.PacketContentType)
            {
                case RequestPacketInfo.PacketContentTypes.Html:
                case RequestPacketInfo.PacketContentTypes.Xml:
                case RequestPacketInfo.PacketContentTypes.Atom:
                case RequestPacketInfo.PacketContentTypes.Json:
                default:
                    // We should handle these content types, but I'll leave that
                    // as an exercise to the reader...
                    break;

                case RequestPacketInfo.PacketContentTypes.UrlEncoded:
                    if (!String.IsNullOrEmpty(Properties.Settings.Default.ResourceText))
                    {
                        code = ResponsePacketInfo.ResponseCodes.Http200; // "OK"
                    }
                    else
                    {
                        code = ResponsePacketInfo.ResponseCodes.Http201; // "Created"
                    }
                    Properties.Settings.Default.ResourceText = requestPacket.PacketContents;
                    Properties.Settings.Default.Save();
                    break;
            }

            return code;
        }

        private ResponsePacketInfo.ResponseCodes ProcessPostVerb(RequestPacketInfo requestPacket)
        {
            // Normally we'd process the representation, extract what we needed
            // to store the created resource, and move on. To simplify things,
            // we'll accept a string in the entity body and leave the individual
            // parsing exercises for another day.
            ResponsePacketInfo.ResponseCodes code = ResponsePacketInfo.ResponseCodes.Http204;
            switch (requestPacket.PacketContentType)
            {
                case RequestPacketInfo.PacketContentTypes.Html:
                case RequestPacketInfo.PacketContentTypes.Xml:
                case RequestPacketInfo.PacketContentTypes.Atom:
                case RequestPacketInfo.PacketContentTypes.Json:
                default:
                    break;

                case RequestPacketInfo.PacketContentTypes.UrlEncoded:
                    code = ResponsePacketInfo.ResponseCodes.Http200; // "OK"
                    Properties.Settings.Default.ResourceText = requestPacket.PacketContents;
                    Properties.Settings.Default.Save();
                    break;
            }

            return code;
        }

        private ResponsePacketInfo.ResponseCodes ProcessDeleteVerb(RequestPacketInfo requestPacket)
        {
            // Normally we'd process the representation, extract what we needed
            // to delete the resource, and move on. To simplify things,
            // we'll accept a string in the entity body and leave the individual
            // parsing exercises for another day.
            ResponsePacketInfo.ResponseCodes code = ResponsePacketInfo.ResponseCodes.Http204;
            switch (requestPacket.PacketContentType)
            {
                case RequestPacketInfo.PacketContentTypes.Html:
                case RequestPacketInfo.PacketContentTypes.Xml:
                case RequestPacketInfo.PacketContentTypes.Atom:
                case RequestPacketInfo.PacketContentTypes.Json:
                default:
                    break;

                case RequestPacketInfo.PacketContentTypes.UrlEncoded:
                    code = ResponsePacketInfo.ResponseCodes.Http200; // "OK"
                    Properties.Settings.Default.ResourceText = String.Empty;
                    Properties.Settings.Default.Save();
                    break;
            }

            return code;
        }

        private string FormatError(ResponsePacketInfo.ResponseCodes errorCode)
        {
            string errorString = String.Empty;
            switch (errorCode)
            {
                case ResponsePacketInfo.ResponseCodes.Http400:
                    errorString = String.Format(ErrorFormat, "Bad Request", "HTTP Error 400: The request was improperly formed.");
                    break;

                case ResponsePacketInfo.ResponseCodes.Http401:
                    errorString = String.Format(ErrorFormat, "Unauthorized", "HTTP Error 401: The requested resource requires authentication.");
                    break;

                case ResponsePacketInfo.ResponseCodes.Http404:
                    errorString = String.Format(ErrorFormat, "Not Found", "HTTP Error 404: The requested resource is not found.");
                    break;

                case ResponsePacketInfo.ResponseCodes.Http405:
                    errorString = String.Format(ErrorFormat, "Method Not Allowed", "HTTP Error 405: The requested action is not allowed.");
                    break;

                case ResponsePacketInfo.ResponseCodes.Http409:
                    errorString = String.Format(ErrorFormat, "Conflict", "HTTP Error 409: The resource already exists. Delete it first, then create a new resource.");
                    break;

                case ResponsePacketInfo.ResponseCodes.Http412:
                    errorString = String.Format(ErrorFormat, "Precondition Failed", "HTTP Error 412: HTTP Basic Authentication is the only acceptable authentication scheme.");
                    break;

                default:
                    errorString = String.Format(ErrorFormat, "Internal Server Error", "HTTP Error 500: Internal server error. Cannot provide additional information.");
                    break;
            }

            return errorString;
        }

        #endregion
    }
}
