﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading;
using System.Net;

namespace HttpExerciser
{
    internal class ListenerWorkUnit
    {
        // Declare a delegate the caller can use to receive HTTP packet
        // information as they arrive.
        public delegate void PacketsProcessedDelegate(RequestPacketInfo requestPacket, ResponsePacketInfo responsePacket);

        // Declare an event that fires when packets are received
        public event PacketsProcessedDelegate PacketsProcessed;

        // An event used to hold the worker thread active until we
        // manually request a shutdown.
        ManualResetEvent _shutdown = new ManualResetEvent(false);

        // Our HttpListener instance
        HttpListener _listener = null;

        // Our assigned prefixes to listen for
        private List<string> _prefixes = null;

        public ListenerWorkUnit(List<string> prefixes)
        {
            this._prefixes = prefixes;
        }

        public void CancelProcessing()
        {
            // Cancel the listener
            _listener.Abort();

            // Trip the shutdown event
            _shutdown.Set();
        }

        public void ProcessPackets(Int32 port)
        {
            // Create a listener
            _listener = new HttpListener();

            // Validate prefixes
            if (this._prefixes.Count == 0)
            {
                throw new ApplicationException("You must provide at least one URI to listen for.");
            }

            // Assign the prefixes
            foreach (string prefix in this._prefixes)
            {
                _listener.Prefixes.Add(prefix);
            }

            // Start the listener
            _listener.Start();

            // Check for secure operations
            if ((bool)Properties.Settings.Default["Secure"])
            {
                // Use secure ops. Note this adds the WWW-Authenticate header
                // for us...
                _listener.AuthenticationSchemes = AuthenticationSchemes.Basic;
            }

            // Listen
            _listener.BeginGetContext(new AsyncCallback(RequestReceived), this);

            // Wait to quit. When the event is set, the
            // listener will already have been aborted, so
            // we don't need to call its Stop method.
            _shutdown.WaitOne();
        }

        private static void RequestReceived(IAsyncResult result)
        {
            // Retrieve the work unit and listener
            ListenerWorkUnit workUnit = (ListenerWorkUnit)result.AsyncState;
            HttpListener listener = workUnit._listener;

            // Check to see if we're still listening (we may have
            // shut down while waiting for this request).
            if (!listener.IsListening)
            {
                // We shut down, so just return...
                return;
            }

            // Yank the context for this request
            HttpListenerContext context = null;
            try
            {
                context = listener.EndGetContext(result);
                if (context == null)
                {
                    throw new Exception();
                }
            }
            catch
            {
                // Sometimes a race condition exists when shutting down,
                // so just return if we throw.
                return;
            }

            // Create a packet processor
            PacketProcessor processor = new PacketProcessor();

            // Interpret incoming request
            RequestPacketInfo requestPacket = processor.ProcessRequest(context.Request);

            // Check to see if authentication is required
            ResponsePacketInfo responsePacket = null;
            if ((bool)Properties.Settings.Default["Secure"])
            {
                try
                {
                    // Authorization is required, so check to see that
                    // the HTTP Authorization header is present.
                    if (requestPacket.ContainsAuthorization)
                    {
                        // The header is present, so let's now parse the header
                        // and check the username/password. Doing so might
                        // throw BadAuthorizationHeaderException or
                        // UnsupportedSchemeException.
                        requestPacket.ParseAuthorization();

                        // Parsing passed, so now check credentials
                        if (requestPacket.Username == (string)Properties.Settings.Default["Username"] &&
                            requestPacket.Password == (string)Properties.Settings.Default["Password"])
                        {
                            // Authenticated...good job!
                            responsePacket = processor.ProcessResponse(requestPacket);
                        }
                        else
                        {
                            // Failed to authenticate...
                            throw new UnauthorizationException();
                        }
                    }
                    else
                    {
                        // Failed to authenticate...
                        throw new UnauthorizationException();
                    }
                }
                catch (BadAuthorizationHeaderException)
                {
                    // Client provided a malformed header
                    responsePacket = processor.ProcessResponse(requestPacket, ResponsePacketInfo.ResponseCodes.Http400);
                }
                catch (UnsupportedSchemeException)
                {
                    // Client provided incorrect scheme
                    responsePacket = processor.ProcessResponse(requestPacket, ResponsePacketInfo.ResponseCodes.Http412);
                }
                catch (UnauthorizationException)
                {
                    // Client requires authentication
                    responsePacket = processor.ProcessResponse(requestPacket, ResponsePacketInfo.ResponseCodes.Http401);
                }
            }
            else
            {
                // Authorization not required, so process normally
                responsePacket = processor.ProcessResponse(requestPacket);
            }

            // Issue response
            processor.IssueResponse(context.Response, responsePacket);
        
            // Raise event to signal receipt of packets
            if (workUnit.PacketsProcessed != null)
            {
                workUnit.PacketsProcessed(requestPacket, responsePacket);
            }

            // Start it all over again...
            listener.BeginGetContext(new AsyncCallback(RequestReceived), workUnit);
        }
    }
}
