﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.Serialization;
using System.Text;
using System.Xml;
using System.Xml.Linq;
using System.Xml.Serialization;

namespace SerializationSampler
{
    [XmlRoot(Namespace = "http://www.scottseely.com/RESTBook/2008")]
    [DataContract(Namespace = "http://www.scottseely.com/RESTBook/2008")]
    public class ImageItem
    {
        [XmlElement(ElementName = "Caption")]
        [DataMember(Name = "Caption")]
        public string Caption { get; set; }

        [XmlElement(ElementName = "Description")]
        [DataMember(Name = "Description")]
        public string Description { get; set; }

        [XmlElement(ElementName = "ImageBytes")]
        [DataMember(Name = "ImageBytes")]
        public byte[] ImageBytes { get; set; }

        [XmlElement(ElementName = "ImageId")]
        [DataMember(Name = "ImageId")]
        public Guid ImageId { get; set; }

        [XmlIgnore]
        [DataMember]
        public Uri ImageUrl { get; set; }

        [XmlElement(ElementName = "ImageUrl")]
        public string ImageUrlString
        {
            get
            {
                return ImageUrl == null ? null : ImageUrl.ToString();
            }
            set
            {
                Uri tempUri = null;
                if (Uri.TryCreate(value, UriKind.Absolute, out tempUri))
                {
                    ImageUrl = tempUri;
                }
                else
                {
                    ImageUrl = null;
                }
            }
        }

        [XmlElement(ElementName = "PublicImage")]
        [DataMember(Name = "PublicImage")]
        public bool PublicImage { get; set; }

        [XmlElement(ElementName = "UserName")]
        [DataMember(Name = "UserName")]
        public string UserName { get; set; }

        public override string ToString()
        {
            return string.Format("\tCaption: {0}{1}" +
                "\tDescription: {2}{1}" +
                "\tImageId: {3}{1}" +
                "\tImageUrl: {4}{1}" +
                "\tPublicImage: {5}{1}" +
                "\tUserName: {6}{1}",
                Caption, Environment.NewLine,
                Description,
                ImageId,
                ImageUrl,
                PublicImage,
                UserName);
        }
    }

    class Program
    {
        const string Data =
            "<ImageItem xmlns=\"http://www.scottseely.com/RESTBook/2008\" "
                + "xmlns:i=\"http://www.w3.org/2001/XMLSchema-instance\">"
                + "<Caption>Some caption</Caption>"
                + "<Description>Some description</Description>"
                + "<ImageBytes>AQIDBAUGAQIDBAUGAQIDBAUGAQIDBAUGAQIDBAUGAQIDBAUG"
                + "</ImageBytes>"
                + "<ImageId>33f741ae-934a-4c77-b7f8-0f316000ab53</ImageId>"
                + "<ImageUrl>"
                + "http://www.scottseely.com/PhotoWeb/Image.ashx"
                + "/33f741ae-934a-4c77-b7f8-0f316000ab53</ImageUrl>"
                + "<PublicImage>true</PublicImage>"
                + "<UserName>restuser</UserName>"
          + "</ImageItem> ";

        static Stream DataStream
        {
            get { return new MemoryStream(Encoding.UTF8.GetBytes(Data)); }
        }

        delegate ImageItem DemoXml();

        static void Main(string[] args)
        {
            List<DemoXml> examples = new List<DemoXml>()
    { UseXmlDocument, 
        UseXDocument, 
        UseXDocumentAlternate,
        UseXmlSerialization,
        UseDataContractSerialization};
            foreach (DemoXml demo in examples)
            {
                Console.WriteLine("{0}: {1}{2}",
                    demo.Method.Name,
                    Environment.NewLine,
                    demo());
            }
        }

        static void ReadToText(XmlReader reader)
        {
            while (reader.Read())
            {
                if (reader.NodeType == XmlNodeType.Text)
                {
                    // Break when we hit a Text node.
                    break;
                }
            }
        }

        private static ImageItem UseXmlDocument()
        {
            ImageItem retval = new ImageItem();

            XmlDocument doc = new XmlDocument();
            doc.Load(DataStream);

            // Setup the Infoset mapping informatoin. 
            XmlNamespaceManager nsMgr = new XmlNamespaceManager(doc.NameTable);
            nsMgr.AddNamespace("item", "http://www.scottseely.com/RESTBook/2008");

            // Only pick the first node named ImageItem.
            // Use SelectNodes to pick ALL.
            XmlNode node = doc.DocumentElement.SelectSingleNode(
                "/item:ImageItem", nsMgr);

            // Look at each node in the document.
            XmlNodeReader reader = new XmlNodeReader(node);
            while (reader.Read())
            {
                // Ignore anything that isn't a start element.
                if (reader.NodeType == XmlNodeType.Element)
                {
                    try
                    {
                        switch (reader.LocalName)
                        {
                            case "Caption":
                                ReadToText(reader);
                                retval.Caption = reader.ReadContentAsString();
                                break;
                            case "Description":
                                ReadToText(reader);
                                retval.Description = reader.ReadContentAsString();
                                break;
                            case "ImageBytes":
                                ReadToText(reader);
                                retval.ImageBytes = Convert.FromBase64String(reader.ReadContentAsString());
                                break;
                            case "ImageId":
                                ReadToText(reader);
                                retval.ImageId = new Guid(reader.ReadContentAsString());
                                break;
                            case "ImageUrl":
                                ReadToText(reader);
                                string tempUri = reader.ReadContentAsString();
                                if (Uri.IsWellFormedUriString(tempUri, UriKind.Absolute))
                                {
                                    retval.ImageUrl = new Uri(tempUri);
                                }
                                break;
                            case "PublicImage":
                                ReadToText(reader);
                                retval.PublicImage = reader.ReadContentAsBoolean();
                                break;
                            case "UserName":
                                ReadToText(reader);
                                retval.UserName = reader.ReadContentAsString();
                                break;
                        }
                    }
                    catch
                    {
                        // Parse failure--do nothing 
                    }
                }
            }
            return retval;
        }

        private static ImageItem UseXDocument()
        {
            XDocument doc = XDocument.Load(new XmlTextReader(DataStream));
            XNamespace ns = "http://www.scottseely.com/RESTBook/2008";
            var items = from imageItemNode in doc.Elements(ns + "ImageItem")
                        select new ImageItem()
                        {
                            Caption = imageItemNode.Elements(ns + "Caption").First().Value,
                            Description = imageItemNode.Elements(ns + "Description").First().Value,
                            ImageBytes = Convert.FromBase64String(imageItemNode.Elements(ns + "ImageBytes").First().Value),
                            ImageId = new Guid(imageItemNode.Elements(ns + "ImageId").First().Value),
                            ImageUrl = new Uri(imageItemNode.Elements(ns + "ImageUrl").First().Value),
                            PublicImage = bool.Parse(imageItemNode.Elements(ns + "PublicImage").First().Value),
                            UserName = imageItemNode.Elements(ns + "UserName").First().Value
                        };
            return items.First();
        }

        static ImageItem UseXDocumentAlternate()
        {
            XDocument doc = XDocument.Load(new XmlTextReader(DataStream));
            XNamespace ns = "http://www.scottseely.com/RESTBook/2008";
            var items = from imageItemNode in doc.Elements(ns + "ImageItem")
                        select new
                        {
                            Caption = imageItemNode.Elements(ns + "Caption").First().Value,
                            Description = imageItemNode.Elements(ns + "Description").First().Value,
                            ImageBytes = imageItemNode.Elements(ns + "ImageBytes").First().Value,
                            ImageId = imageItemNode.Elements(ns + "ImageId").First().Value,
                            ImageUrl = imageItemNode.Elements(ns + "ImageUrl").First().Value,
                            PublicImage = imageItemNode.Elements(ns + "PublicImage").First().Value,
                            UserName = imageItemNode.Elements(ns + "UserName").First().Value
                        };
            ImageItem retval = new ImageItem();
            foreach (var item in items)
            {
                retval.Caption = item.Caption;
                retval.Description = item.Description;
                retval.UserName = item.UserName;
                bool publicImage;
                if (bool.TryParse(item.PublicImage, out publicImage))
                {
                    retval.PublicImage = publicImage;
                }
                if (Uri.IsWellFormedUriString(item.ImageUrl, UriKind.Absolute))
                {
                    retval.ImageUrl = new Uri(item.ImageUrl);
                }
                try
                {
                    retval.ImageId = new Guid(item.ImageId);
                }
                catch { }
                try
                {
                    retval.ImageBytes = Convert.FromBase64String(item.ImageBytes);
                }
                catch { }
                break;
            }
            return retval;
        }

        static ImageItem UseXmlSerialization()
        {
            XmlSerializer ser = new XmlSerializer(typeof(ImageItem));
            return (ImageItem)ser.Deserialize(DataStream);
        }

        static ImageItem UseDataContractSerialization()
        {
            DataContractSerializer dcs = new DataContractSerializer(typeof(ImageItem));
            return (ImageItem)dcs.ReadObject(DataStream);
        }

    }
}
