﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Net;
using System.Runtime.Serialization;
using System.Threading;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media.Imaging;
using Microsoft.Win32;
using PhotoShared;

namespace WpfPhotoClient
{
    /// <summary>
    /// Interaction logic for ImageView.xaml
    /// </summary>
    public partial class ImageView : Window
    {
        public string LoggedInUserId { get; set; }
        public string CurrentUserId { get; set; }
        public string LoginToken { get; set; }
        List<ImageItem> ImageDocument { get; set; }
        delegate void PlainMethod();
        public ImageView()
        {
            InitializeComponent();
        }

        bool ItemChanged { get; set; }

        private void btnLoadFriend_Click(object sender, RoutedEventArgs e)
        {
            CurrentUserId = txtFriendName.Text.Trim();
            LoadImages();
        }

        private void btnLoadMyImages_Click(object sender, RoutedEventArgs e)
        {
            CurrentUserId = LoggedInUserId;
            txtFriendName.Text = string.Empty;
            LoadImages();
        }

        private void btnAddImage_Click(object sender, RoutedEventArgs e)
        {
            OpenFileDialog ofd = new OpenFileDialog();
            ofd.DefaultExt = "Image files|*.jpg;*.jpeg;*.gif;*.png;*.bmp|All files|*.*";
            ofd.Multiselect = true;
            if (ofd.ShowDialog(this).Value)
            {
                this.IsEnabled = false;
                ThreadPool.QueueUserWorkItem(new WaitCallback(AddImages), ofd.FileNames);
            }
        }

        private void btnDeleteImage_Click(object sender, RoutedEventArgs e)
        {
            for (int i = lstImages.SelectedItems.Count - 1; i >= 0; --i)
            {
                ThreadPool.QueueUserWorkItem(new WaitCallback(DeleteImage),
                    ((ListViewItem)lstImages.SelectedItems[i]).Tag);
                lstImages.Items.Remove(lstImages.SelectedItems[i]);
            }
        }

        void DeleteImage(object data)
        {
            ImageItem imageItem = (ImageItem)data;
            if (data == null)
            {
                return;
            }
            Uri callUri = Utility.CreateUri(ServiceType.Image, string.Format("Image/{0}",
                imageItem.ImageId));
            WebRequest request = Utility.CreateRequest(callUri);
            request.ContentType = "application/xml";
            request.Method = "DELETE";
            try
            {
                WebResponse response = request.GetResponse();
            }
            catch (WebException)
            {
                Dispatcher.Invoke((PlainMethod)delegate
                {
                    MessageBox.Show(this, "Failed to delete an image. Try logging back in again.", "Delete Failed", MessageBoxButton.OK, MessageBoxImage.Error);
                });
            }
        }

        void AddImages(object data)
        {
            string[] filenames = (string[])data;
            DataContractSerializer itemSerializer = new DataContractSerializer(typeof(ImageItem));
            foreach (string filename in filenames)
            {
                Uri callUri = Utility.CreateUri(ServiceType.Image, "AddImage/");
                WebRequest request = Utility.CreateRequest(callUri);
                request.ContentType = "application/xml";
                request.Method = "POST";
                Stream requestStream = request.GetRequestStream();
                ImageItem imageItem = new ImageItem()
                {
                    PublicImage = false,
                    ImageBytes = File.ReadAllBytes(filename),
                    UserName = LoggedInUserId
                };
                itemSerializer.WriteObject(requestStream, imageItem);
                requestStream.Close();
                try
                {
                    WebResponse response = request.GetResponse();
                    DataContractSerializer guidSerializer = new DataContractSerializer(typeof(Guid));
                    Guid lastImageId = (Guid)guidSerializer.ReadObject(response.GetResponseStream());

                    if (lastImageId != Guid.Empty)
                    {
                        imageItem.ImageId = lastImageId;
                        ImageDocument.Add(imageItem);
                        this.Dispatcher.Invoke((PlainMethod)delegate
                        {
                            StackPanel sp = new StackPanel();
                            BitmapImage image = new BitmapImage(new Uri(filename));
                            sp.Children.Add(new Image() { Source = image, Height = 100, Width = 100 });
                            sp.Children.Add(new Label() { Content = imageItem.Caption });
                            lstImages.Items.Add(new ListViewItem() { Content = sp, Tag = imageItem });
                        });
                    }
                }
                catch (WebException)
                {
                    Dispatcher.Invoke((PlainMethod)delegate
                    {
                        MessageBox.Show(this, "Failed to add an image. Try logging back in again.", "Add Failed", MessageBoxButton.OK, MessageBoxImage.Error);
                    });
                }
            }
            this.Dispatcher.Invoke((PlainMethod)delegate
            {
                this.IsEnabled = true;
            });
        }

        void LoadImages()
        {
            Uri imagesUri = null;
            lstImages.Items.Clear();
            imagesUri = Utility.CreateUri(ServiceType.Image,
                string.Format("Images/{0}", CurrentUserId));

            // Update the UI. If the user isn't the owner of the images,
            // disable the editable fields.
            SetDescriptionFields(!string.Equals(CurrentUserId, LoggedInUserId,
                StringComparison.OrdinalIgnoreCase));

            // Get the list of images.
            WebRequest request = Utility.CreateRequest(imagesUri);
            request.BeginGetResponse(new AsyncCallback(ImagesResponse), request);
        }

        void SetDescriptionFields(bool readOnly)
        {
            chkPublic.IsEnabled = !readOnly;
            btnDeleteImage.Visibility = readOnly ? Visibility.Hidden : Visibility.Visible;
            txtCaption.IsEnabled = !readOnly;
            txtDescription.IsEnabled = !readOnly;
        }

        void ImagesResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                WebResponse response = request.EndGetResponse(result);
                DataContractSerializer serializer = new DataContractSerializer(typeof(ImageItem[]));
                ImageDocument = new List<ImageItem>((ImageItem[])serializer.ReadObject(response.GetResponseStream()));
                response.Close();
                foreach (ImageItem imageItem in ImageDocument)
                {
                    Dispatcher.Invoke((PlainMethod)delegate
                    {
                        BitmapImage image = new BitmapImage(new Uri(imageItem.ImageUrl));
                        StackPanel sp = new StackPanel();
                        sp.Children.Add(new Image() { Source = image, Height = 100, Width = 100 });
                        sp.Children.Add(new Label() { Content = imageItem.Caption });
                        lstImages.Items.Add(new ListViewItem() { Content = sp, Tag = imageItem });
                    });
                }
            }
            catch (WebException we)
            {
                HttpWebResponse response = (HttpWebResponse)we.Response;
                switch (response.StatusCode)
                {
                    case HttpStatusCode.Unauthorized:
                        Dispatcher.Invoke((PlainMethod)delegate
                        {
                            MessageBox.Show(this, "Your credentials are incorrect.", "Incorrect credentials", MessageBoxButton.OK,
                                MessageBoxImage.Stop);
                        });
                        break;
                }
                response.Close();
            }
        }

        public void ImageResponse(IAsyncResult result)
        {
            object[] values = (object[])(result.AsyncState);
            WebRequest request = (WebRequest)values[0];
            ImageItem imageItem = (ImageItem)values[1];
            try
            {
                WebResponse response = request.EndGetResponse(result);
                this.Dispatcher.Invoke((PlainMethod)delegate
                {
                    BitmapImage image = new BitmapImage();
                    image.CacheOption = BitmapCacheOption.OnLoad;
                    image.BeginInit();
                    image.StreamSource = response.GetResponseStream();
                    image.EndInit();
                    image.StreamSource.Close();
                    StackPanel sp = new StackPanel();
                    sp.Children.Add(new Image() { Source = image, Height = 100, Width = 100 });
                    sp.Children.Add(new Label() { Content = imageItem.Caption });
                    lstImages.Items.Add(new ListViewItem() { Content = sp, Tag = imageItem });
                });
            }
            catch (Exception we)
            {
                this.Dispatcher.Invoke((PlainMethod)delegate
                {
                    MessageBox.Show(we.ToString());
                });
            }
        }
        
        private void ShowImages(object dummy)
        {
            foreach (ImageItem imageItem in ImageDocument)
            {
                // This is executing on a background thread, so we block 
                // and wait for each image to load. 
                BitmapImage image = new BitmapImage(new Uri(imageItem.ImageUrl));

                // Add the image to the screen.
                this.Dispatcher.Invoke((PlainMethod)delegate
                {
                    StackPanel sp = new StackPanel();
                    sp.Children.Add(new Image() { Source = image, Height = 100, Width = 100 });
                    sp.Children.Add(new Label() { Content = imageItem.Caption });
                    lstImages.Items.Add(new ListViewItem() { Content = sp, Tag = imageItem });
                });
            }
        }

        private void Window_Loaded(object sender, RoutedEventArgs e)
        {
            if (Properties.Settings.Default.LoadMyImages)
            {
                btnLoadMyImages_Click(this, null);
            }
        }

        private void lstImages_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (string.Equals(CurrentUserId, LoggedInUserId, StringComparison.OrdinalIgnoreCase))
            {
                if (ItemChanged && e.RemovedItems.Count > 0)
                {
                    ListViewItem listViewItem = (ListViewItem)e.RemovedItems[0];
                    ImageItem item = (ImageItem)listViewItem.Tag;
                    ((Label)(((StackPanel)listViewItem.Content).Children[1])).Content = item.Caption;
                    ThreadPool.QueueUserWorkItem(new WaitCallback(UpdateImage), item);
                }
            }
            ItemChanged = false;
        }

        void UpdateImage(object data)
        {
            ImageItem imageItem = (ImageItem)data;
            if (data == null)
            {
                return;
            }
            DataContractSerializer itemSerializer = new DataContractSerializer(typeof(ImageItem));
            Uri callUri = Utility.CreateUri(ServiceType.Image, string.Format("Image/{0}",
                imageItem.ImageId));
            WebRequest request = Utility.CreateRequest(callUri);
            request.ContentType = "application/xml";
            request.Method = "PUT";
            Stream requestStream = request.GetRequestStream();
            itemSerializer.WriteObject(requestStream, imageItem);
            requestStream.Close();
            request.BeginGetResponse(UpdateResponse, request);
        }

        void UpdateResponse(IAsyncResult result)
        {
            WebRequest request = (WebRequest)result.AsyncState;
            try
            {
                request.EndGetResponse(result).Close();
            }
            catch (WebException we)
            {
                Debug.WriteLine(we.Message);
            }
        }
        
        private void chkPublic_Checked(object sender, RoutedEventArgs e)
        {
            ItemChanged = true;
        }

        private void txtCaption_TextChanged(object sender, TextChangedEventArgs e)
        {
            ItemChanged = true;
        }

        private void txtDescription_TextChanged(object sender, TextChangedEventArgs e)
        {
            ItemChanged = true;
        }
    }
}
