﻿// [Book name], ISBN: [isbn]
// Kenn Scribner and Scott Seely
// This code supplied without any warranties, express or implied. 
// Code is free to use, public domain. 
// Have a nice day!

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Threading;
using System.Web;

namespace RESTExtensions
{
    public class ProcessCacheModule : IHttpModule
    {
        ReaderWriterLock _cacheLock = new ReaderWriterLock();
        List<CacheItem> _cache = new List<CacheItem>();

        static readonly TimeSpan MaxAge = TimeSpan.FromSeconds(30);
        static readonly TimeSpan ReadLockTimeout = TimeSpan.FromSeconds(1);
        static readonly TimeSpan WriteLockTimeout = TimeSpan.FromSeconds(5);
        const int MaxCacheItems = 5;

        public void Dispose() { }

        public void Init(HttpApplication context)
        {
            context.ResolveRequestCache += ResolveRequestCacheHandler;
            context.UpdateRequestCache += UpdateRequestCacheHandler;
        }


        void ResolveRequestCacheHandler(object sender, EventArgs e)
        {
            _cacheLock.AcquireReaderLock(ReadLockTimeout);
            bool responseWritten = false;
            HttpResponse response = HttpContext.Current.Response;
            if (_cacheLock.IsReaderLockHeld)
            {
                try
                {
                    string requestUrl = 
                        HttpContext.Current.Request.Url.ToString();
                    var cacheItems = from CacheItem cache in _cache
                                     where
                                         string.Equals(cache.Url, requestUrl, 
                                         StringComparison.OrdinalIgnoreCase)
                                     select cache;
                    foreach (CacheItem cacheItem in cacheItems)
                    {
                        if ((cacheItem.Created + MaxAge) >= DateTime.UtcNow)
                        {
                            // Only write the response if it is recent.
                            // Otherwise, the slot will be fixed up at the end.
                            // It is possible for two requests for the same URL 
                            // to exit through here and hit the handler. This 
                            // will be resolved when the cache is updated.
                            response.OutputStream.Write(cacheItem.Response,
                                0, cacheItem.Response.Length);
                            response.ContentType = "text/xml";
                            responseWritten = true;
                            response.End();
                        }
                        break;
                    }
                }
                finally
                {
                    _cacheLock.ReleaseReaderLock();
                }
            }
            if (!responseWritten)
            {
                // If item was not found in cache, add a filter
                // that can capture the output and put the 
                // response into the cache.
                response.Filter = new CacheStream(response.Filter);
            }
        }

        void UpdateRequestCacheHandler(object sender, EventArgs e)
        {
            HttpResponse response = HttpContext.Current.Response;
            if (response.StatusCode == (int)HttpStatusCode.OK)
            {
                _cacheLock.AcquireWriterLock(WriteLockTimeout);
                if (_cacheLock.IsWriterLockHeld)
                {
                    try
                    {
                        string requestUrl =
                           HttpContext.Current.Request.Url.ToString();
                        for (int i = _cache.Count - 1; i >= 0; --i)
                        {
                            CacheItem cacheItem = _cache[i];

                            // Look for an item that is either old or that 
                            // matches the current request.
                            if ((
                             (cacheItem.Created + MaxAge) <= DateTime.UtcNow) ||
                             string.Equals(cacheItem.Url, requestUrl,
                                StringComparison.OrdinalIgnoreCase))
                            {
                                _cache.RemoveAt(i);
                            }
                            break;
                        }
                        while (_cache.Count >= MaxCacheItems)
                        {
                            _cache.RemoveAt(_cache.Count - 1);
                        }
                        CacheStream cacheStream = response.Filter as
                            CacheStream;
                        if (cacheStream != null)
                        {
                            cacheStream.Cache.Flush();
                            byte[] memoryBuffer = cacheStream.Cache.GetBuffer();
                            byte[] buffer = new byte[memoryBuffer.Length];
                            memoryBuffer.CopyTo(buffer, 0);
                            // Insert at the head
                            _cache.Insert(0, new CacheItem(requestUrl, buffer));
                        }
                    }
                    catch (Exception ex)
                    {
                        Debug.WriteLine(ex.ToString());
                        throw ex;
                    }
                    finally
                    {
                        _cacheLock.ReleaseWriterLock();
                    }
                }
            }
            // If the cache could not be acquired, keep on going anyhow.
        }
    }
}
