﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Net;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Web;
using System.IO;

namespace AspNetRestService
{
    public class BlogService : IHttpHandler
    {
        #region Constants

        const string HttpHeadMethod = "head";
        const string HttpGetMethod = "get";
        const string HttpPostMethod = "post";
        const string HttpPutMethod = "put";
        const string HttpDeleteMethod = "delete";

        #endregion

        delegate void HandleRequest(HttpContext context, UriTemplateMatch template);
        Model _model = new Model();
        DataContractSerializer _blogXmlItemSerializer =
            new DataContractSerializer(typeof(BlogEntryItem));
        DataContractJsonSerializer _blogJsonItemSerializer =
            new DataContractJsonSerializer(typeof(BlogEntryItem));
        DataContractSerializer _blogXmlCommentSerializer =
            new DataContractSerializer(typeof(BlogCommentItem));
        DataContractJsonSerializer _blogJsonCommentSerializer =
            new DataContractJsonSerializer(typeof(BlogCommentItem));
        DataContractSerializer _blogXmlListSerializer =
            new DataContractSerializer(typeof(IEnumerable<BlogEntryItem>));
        DataContractJsonSerializer _blogJsonListSerializer =
            new DataContractJsonSerializer(typeof(IEnumerable<BlogEntryItem>));
        UriTemplateTable _templateTable = null;
        object _tableLock = new object();

        UriTemplateTable TemplateTable
        {
            get
            {
                if (_templateTable == null)
                {
                    lock (_tableLock)
                    {
                        if (_templateTable == null)
                        {
                            HttpRequest request = HttpContext.Current.Request;
                            string fullUrl = request.Url.ToString();
                            string baseUrl = fullUrl.Substring(0, fullUrl.IndexOf(
                                request.ApplicationPath));
                            _templateTable = new UriTemplateTable();
                            _templateTable.BaseAddress = new Uri(baseUrl);
                            _templateTable.KeyValuePairs.Add(
                                new KeyValuePair<UriTemplate, object>(
                                 new UriTemplate(
                                     request.ApplicationPath +
                                     "/Blog"), new HandleRequest(HandleBlogList)));
                            _templateTable.KeyValuePairs.Add(
                                new KeyValuePair<UriTemplate, object>(
                                    new UriTemplate(request.ApplicationPath +
                                        "/Blog/{BlogID}"),
                                        new HandleRequest(HandleBlogItem)));
                        }
                    }
                }
                return _templateTable;
            }
        }

        public bool IsReusable
        {
            get { return true; }
        }

        public void ProcessRequest(HttpContext context)
        {
            UriTemplateMatch match = TemplateTable.MatchSingle(context.Request.Url);
            if (match == null)
            {
                context.Response.StatusCode = (Int32)HttpStatusCode.NotFound;
            }
            else
            {
                HandleRequest handleRequest = (HandleRequest)match.Data;
                handleRequest(context, match);
            }
        }

        private void HandleBlogList(HttpContext context, UriTemplateMatch template)
        {
            // Check HTTP method and perform appropriate action. For the
            // /Blog URI, we only allow HEAD, GET, and POST.
            switch (context.Request.HttpMethod.ToLower())
            {
                case HttpHeadMethod:
                    DispatchGetBlogList(context, false);
                    break;

                case HttpGetMethod:
                    DispatchGetBlogList(context, true);
                    break;

                case HttpPostMethod:
                    DispatchAddBlogItem(context);
                    break;

                default:
                    DispatchNotAllowed(context);
                    break;
            }
        }

        private void HandleBlogItem(HttpContext context, UriTemplateMatch template)
        {
            // Interpret the blog ID
            Int32 blogID = 0;
            if (!Int32.TryParse(template.BoundVariables["BlogID"], out blogID))
            {
                // Invalid blog ID, so return error
                DispatchBadRequest(context);
                return;
            }

            // Assign the blog ID to the Items collection in the context
            // so we don't have to pass it around as a separate value.
            context.Items.Add("BlogID", blogID);

            // Check HTTP method and perform appropriate action. For the
            // /Blog/{BlogID} URI, we allow HEAD, GET, POST, PUT, and DELETE.
            switch (context.Request.HttpMethod.ToLower())
            {
                case HttpHeadMethod:
                    DispatchGetBlogItem(context, false);
                    break;

                case HttpGetMethod:
                    DispatchGetBlogItem(context, true);
                    break;

                case HttpPutMethod:
                    DispatchUpdateBlogItem(context);
                    break;

                case HttpPostMethod:
                    DispatchAddBlogComment(context);
                    break;

                case HttpDeleteMethod:
                    DispatchDeleteBlogItem(context);
                    break;

                default:
                    DispatchNotAllowed(context);
                    break;
            }
        }

        private void DispatchNotAllowed(HttpContext context)
        {
            // Return method not allowed
            context.Response.StatusCode = (Int32)HttpStatusCode.MethodNotAllowed;
        }

        private void DispatchBadRequest(HttpContext context)
        {
            // Malformed request
            context.Response.StatusCode = (Int32)HttpStatusCode.BadRequest;
        }

        private void DispatchGetBlogList(HttpContext context, bool issueContent)
        {
            // Test the user's credentials
            if (context.User.IsInRole("Read"))
            {
                // Retrieve all blog entries
                IEnumerable<BlogEntryItem> blogItems = _model.QueryBlog();

                // Determine stream to write to. If we're issuing content, we'll
                // write to the output stream. If a HEAD request, we'll write to
                // a temporary stream so we can determine the content length.
                Stream outputStream = context.Response.OutputStream;
                if (!issueContent)
                {
                    // Create a temporary stream
                    outputStream = new MemoryStream();
                }

                if (context.Request.Headers["Accept"].ToLower() == "application/json")
                {
                    // Notify caller that the response resource is in JSON.
                    context.Response.ContentType = "application/json";
                    _blogJsonListSerializer.WriteObject(
                        outputStream, blogItems);
                }
                else
                {
                    // Notify caller that the response resource is in XML.
                    context.Response.ContentType = "text/xml";
                    _blogXmlListSerializer.WriteObject(
                        outputStream, blogItems);
                }

                // If we're not issuing content, we'll need to set the content
                // length header ourselves...
                if (!issueContent)
                {
                    // Set the content length header
                    context.Response.Headers["Content-Length"] = outputStream.Length.ToString();
                }
            }
            else
            {
                // Return forbidden
                context.Response.StatusCode = (Int32)HttpStatusCode.Forbidden;
            }
        }

        private void DispatchAddBlogItem(HttpContext context)
        {
            // Test the user's credentials
            if (context.User.IsInRole("Create"))
            {
                // Extract the representation from the request stream.
                // Note we check the Content-Type header for the type of
                // information we're to deserialize (we accept XML and
                // JSON).
                BlogEntryItem item = null;
                if (context.Request.Headers["Content-Type"].ToLower() == "application/json")
                {
                    // Deserialize the incoming JSON stream
                    item = _blogJsonItemSerializer.ReadObject(
                        context.Request.InputStream) as BlogEntryItem;
                }
                else
                {
                    // Deserialize the incoming XML stream
                    item = _blogXmlItemSerializer.ReadObject(
                        context.Request.InputStream) as BlogEntryItem;
                }

                // If the object was deserialized, insert it into
                // the database.
                BlogEntry entry = _model.InsertBlogItem(item);

                // We now have several tasks we need to perform:
                //
                // 1) Serialize the object to be returned in the
                //    response stream, which represents the object
                //    as inserted into the database.
                // 2) Return 204 (Created)
                // 3) Create a Location header and fill it with the
                //    URI that will access the blog entry.
                if (context.Request.Headers["Accept"].ToLower() == "application/json")
                {
                    // Serialize the new object for return
                    item = new BlogEntryItem(entry, null);
                    _blogJsonItemSerializer.WriteObject(
                        context.Response.OutputStream, item);

                    // Set the return type
                    context.Response.ContentType = "application/json";
                }
                else
                {
                    // Serialize the new object for return
                    item = new BlogEntryItem(entry, null);
                    _blogXmlItemSerializer.WriteObject(
                        context.Response.OutputStream, item);

                    // Set the return type
                    context.Response.ContentType = "text/xml";
                }

                // Return created status
                context.Response.StatusCode = (Int32)HttpStatusCode.Created;

                // Create the URI that would access the new item
                string fullUrl = context.Request.Url.ToString();
                string baseUrl = fullUrl.Substring(0, fullUrl.IndexOf(
                    context.Request.ApplicationPath));
                string locationValue = String.Format("{0}{1}/Default.aspx?BlogID={2}",
                    baseUrl, context.Request.ApplicationPath, entry.EntryID);
                context.Response.AddHeader("Location", locationValue);
            }
            else
            {
                // Return forbidden
                context.Response.StatusCode = (Int32)HttpStatusCode.Forbidden;
            }
        }

        private void DispatchGetBlogItem(HttpContext context, bool issueContent)
        {
            // Test the user's credentials
            if (context.User.IsInRole("Read"))
            {
                // Retrieve the item
                BlogEntryItem blogItem = _model.QueryBlogItem((Int32)context.Items["BlogID"]);

                // Determine stream to write to. If we're issuing content, we'll
                // write to the output stream. If a HEAD request, we'll write to
                // a temporary stream so we can determine the content length.
                Stream outputStream = context.Response.OutputStream;
                if (!issueContent)
                {
                    // Create a temporary stream
                    outputStream = new MemoryStream();
                }

                if (context.Request.Headers["Accept"].ToLower() == "application/json")
                {
                    // Notify caller that the response resource is in JSON.
                    context.Response.ContentType = "application/json";
                    _blogJsonItemSerializer.WriteObject(
                        outputStream, blogItem);
                }
                else
                {
                    // Notify caller that the response resource is in XML.
                    context.Response.ContentType = "text/xml";
                    _blogXmlItemSerializer.WriteObject(
                        outputStream, blogItem);
                }

                // If we're not issuing content, we'll need to set the content
                // length header ourselves...
                if (!issueContent)
                {
                    // Set the content length header
                    context.Response.Headers["Content-Length"] = outputStream.Length.ToString();
                }
            }
            else
            {
                // Return forbidden
                context.Response.StatusCode = (Int32)HttpStatusCode.Forbidden;
            }
        }

        private void DispatchUpdateBlogItem(HttpContext context)
        {
            // Test the user's credentials
            if (context.User.IsInRole("Modify"))
            {
                try
                {
                    // Extract the representation from the request stream.
                    // Note we check the Content-Type header for the type of
                    // information we're to deserialize (we accept XML and
                    // JSON).
                    BlogEntryItem candidateItem = null;
                    if (context.Request.Headers["Content-Type"].ToLower() == "application/json")
                    {
                        // Deserialize the incoming JSON stream
                        candidateItem = _blogJsonItemSerializer.ReadObject(
                            context.Request.InputStream) as BlogEntryItem;
                    }
                    else
                    {
                        // Deserialize the incoming XML stream
                        candidateItem = _blogXmlItemSerializer.ReadObject(
                            context.Request.InputStream) as BlogEntryItem;
                    }

                    // Update the data store
                    _model.UpdateBlogItem((Int32)context.Items["BlogID"],
                        candidateItem.Title,
                        candidateItem.Entry);

                    // With the entity updated, simply return OK
                    context.Response.StatusCode = (Int32)HttpStatusCode.OK;
                }
                catch (InvalidResourceException)
                {
                    // Return 400 (Bad Request) since they can't update
                    // the resource (most likely due not finding the
                    // requested item in the database).
                    context.Response.StatusCode = (Int32)HttpStatusCode.BadRequest;
                }
            }
            else
            {
                // Return forbidden
                context.Response.StatusCode = (Int32)HttpStatusCode.Forbidden;
            }
        }

        private void DispatchAddBlogComment(HttpContext context)
        {
            // NOTE: this method assumes the content of the comment has been
            // reviewed and approved through some workflow process not shown
            // here. It is not advisable to simply post comments without
            // reviewing them first. Also consider SQL Injection attacks and
            // preventive measures.
            // 
            // Test the user's credentials
            if (context.User.IsInRole("Create"))
            {
                // Extract the representation from the request stream.
                // Note we check the Content-Type header for the type of
                // information we're to deserialize (we accept XML and
                // JSON).
                BlogCommentItem item = null;
                if (context.Request.Headers["Content-Type"].ToLower() == "application/json")
                {
                    // Deserialize the incoming JSON stream
                    item = _blogJsonCommentSerializer.ReadObject(
                        context.Request.InputStream) as BlogCommentItem;
                }
                else
                {
                    // Deserialize the incoming XML stream
                    item = _blogXmlCommentSerializer.ReadObject(
                        context.Request.InputStream) as BlogCommentItem;
                }

                // If the object was deserialized, insert it into
                // the database.
                BlogComment comment = _model.InsertBlogComment((Int32)context.Items["BlogID"], item);

                // We now have several tasks we need to perform:
                //
                // 1) Serialize the object to be returned in the
                //    response stream, which represents the object
                //    as inserted into the database.
                // 2) Return 204 (Created)
                // 3) Create a Location header and fill it with the
                //    URI that will access the blog entry.
                if (context.Request.Headers["Accept"].ToLower() == "application/json")
                {
                    // Serialize the new object for return
                    item = new BlogCommentItem(comment);
                    _blogJsonCommentSerializer.WriteObject(
                        context.Response.OutputStream, item);

                    // Set the return type
                    context.Response.ContentType = "application/json";
                }
                else
                {
                    // Serialize the new object for return
                    item = new BlogCommentItem(comment);
                    _blogXmlCommentSerializer.WriteObject(
                        context.Response.OutputStream, item);

                    // Set the return type
                    context.Response.ContentType = "text/xml";
                }

                // Return created status
                context.Response.StatusCode = (Int32)HttpStatusCode.Created;

                // Create the URI that would access the new item
                string fullUrl = context.Request.Url.ToString();
                string baseUrl = fullUrl.Substring(0, fullUrl.IndexOf(
                    context.Request.ApplicationPath));
                string locationValue = String.Format("{0}{1}/Default.aspx?BlogID={2}",
                    baseUrl, context.Request.ApplicationPath, comment.EntryID);
                context.Response.AddHeader("Location", locationValue);
            }
            else
            {
                // Return forbidden
                context.Response.StatusCode = (Int32)HttpStatusCode.Forbidden;
            }
        }

        private void DispatchDeleteBlogItem(HttpContext context)
        {
            // Test the user's credentials
            if (context.User.IsInRole("Delete"))
            {
                // Delete the item
                _model.DeleteBlogItem((Int32)context.Items["BlogID"]);

                // Since we're not returning an entity in the response
                // stream, RFC 2616 dictates we return 204 (NoContent).
                context.Response.StatusCode = (Int32)HttpStatusCode.NoContent;
            }
            else
            {
                // Return forbidden
                context.Response.StatusCode = (Int32)HttpStatusCode.Forbidden;
            }
        }
    }
}
