﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Transactions;

namespace AspNetRestService
{
    public class Model
    {
        public IEnumerable<BlogEntryItem> QueryBlog()
        {
            // Retrieve all blog entries
            BlogDataClassesDataContext context = new BlogDataClassesDataContext();
            var blogQuery = from b in context.BlogEntries orderby b.PostDate select b;
            List<BlogEntryItem> entries = new List<BlogEntryItem>();
            foreach (var entry in blogQuery)
            {
                // Create the new comment item
                BlogEntryItem entrytItem = new BlogEntryItem(entry, null);
                entries.Add(entrytItem);
            }
            return entries;
        }

        public BlogEntry InsertBlogItem(BlogEntryItem item)
        {
            // Create the data context
            BlogDataClassesDataContext context = new BlogDataClassesDataContext();

            // Create the entry into the blog table
            BlogEntry entry = new BlogEntry();
            entry.Title = item.Title;
            entry.Entry = item.Entry;
            entry.PostDate = DateTime.Now;

            // Schedule the new entry for insertion
            context.BlogEntries.InsertOnSubmit(entry);

            // Transact the blog entry into the database
            using (TransactionScope ts = new TransactionScope())
            {
                // Submit the entry
                context.SubmitChanges();

                // Commit the insert
                ts.Complete();
            }

            // The blog entry ID is updated on insert, so we'll return
            // the updated object so that we can create the response
            // to the caller.
            return entry;
        }

        public void UpdateBlogItem(Int32 blogID, string title, string entry)
        {
            // Create the data context
            BlogDataClassesDataContext context = new BlogDataClassesDataContext();
            var blogQuery = from b in context.BlogEntries where b.EntryID == blogID select b;

            // Update the item. Note if the item doesn't exist, update via
            // HTTP PUT is invalid (since the client can't dictate the URI),
            // so the exception handler will return the appropriate error.
            if (blogQuery.Count() > 0)
            {
                blogQuery.First().Title = title;
                blogQuery.First().Entry = entry;

                // Transact the blog entry into the database
                using (TransactionScope ts = new TransactionScope())
                {
                    // Submit the entry
                    context.SubmitChanges();

                    // Commit the insert
                    ts.Complete();
                }
            }
            else
            {
                // Can't find the resource
                throw new InvalidResourceException();
            }
        }

        public void DeleteBlogItem(Int32 blogID)
        {
            // Create the data context
            BlogDataClassesDataContext context = new BlogDataClassesDataContext();
            var blogQuery = from b in context.BlogEntries where b.EntryID == blogID select b;
            var commentQuery = from c in context.BlogComments where c.EntryID == blogID select c;

            // Delete the item. Note if it doesn't exist, we won't
            // return any sort of error. We'll just return. From
            // a data management perspective, we might prefer to set
            // a bit in the record that says "deleted," but for 
            // demonstration purposes I'll actually delete the
            // blog record and any related comments.
            if (blogQuery.Count() > 0)
            {
                context.BlogEntries.DeleteAllOnSubmit(blogQuery);
                if (commentQuery.Count() > 0)
                {
                    // Don't want to orphan any comments...
                    context.BlogComments.DeleteAllOnSubmit(commentQuery);
                }

                // Transact the delete in the database
                using (TransactionScope ts = new TransactionScope())
                {
                    // Submit the entry
                    context.SubmitChanges();

                    // Commit the insert
                    ts.Complete();
                }
            }
        }

        public BlogEntryItem QueryBlogItem(Int32 blogID)
        {
            // Retrieve the blog entry
            BlogDataClassesDataContext context = new BlogDataClassesDataContext();
            var blogQuery = from b in context.BlogEntries where b.EntryID == blogID select b;

            // Retrieve the comments
            var commentQuery = from c in context.BlogComments where c.EntryID == blogID orderby c.PostDate select c;

            // Create the comment array
            List<BlogCommentItem> comments = new List<BlogCommentItem>();
            foreach (var comment in commentQuery)
            {
                // Create the new comment item
                BlogCommentItem commentItem = new BlogCommentItem(comment);
                comments.Add(commentItem);
            }

            // Create returned item
            BlogEntryItem item = new BlogEntryItem(blogQuery.First(), comments.ToArray());
            return item;
        }

        public BlogComment InsertBlogComment(Int32 blogID, BlogCommentItem rawComment)
        {
            // Create the data context
            BlogDataClassesDataContext context = new BlogDataClassesDataContext();

            // Create the comment into the blog comment table
            BlogComment comment = new BlogComment();
            comment.Comment = rawComment.Comment;
            comment.Commentor = rawComment.Commentor;
            comment.EntryID = blogID;
            comment.PostDate = DateTime.Now;

            // Schedule the new comment for insertion
            context.BlogComments.InsertOnSubmit(comment);

            // Transact the blog entry into the database
            using (TransactionScope ts = new TransactionScope())
            {
                // Submit the entry
                context.SubmitChanges();

                // Commit the insert
                ts.Complete();
            }

            // The blog comment ID is updated on insert, so we'll return
            // the updated object so that we can create the response
            // to the caller.
            return comment;
        }
    }
}
