﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Net;
using System.IO;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Json;
using System.Xml;

namespace WpfClientApplication
{
    /// <summary>
    /// Interaction logic for Window1.xaml
    /// </summary>
    public partial class Window1 : Window
    {
        string _username = String.Empty;
        string _password = String.Empty;
        bool _rememberMe = false;

        public Window1()
        {
            InitializeComponent();
        }

        private void Execute_Click(object sender, RoutedEventArgs e)
        {
            // Check for a URI...we can't let the user progress without
            // one...
            if (String.IsNullOrEmpty(tbServiceUri.Text))
            {
                // No URI...
                MessageBox.Show("You must provide a URI before executing the service call.", "URI Required", MessageBoxButton.OK, MessageBoxImage.Error);
                return;
            }

            BlogEntryItem newEntryItem = null;
            BlogCommentItem newCommentItem = null;
            if (cmbMethod.SelectedIndex == 2 || cmbMethod.SelectedIndex == 3)
            {
                // Determine whether we have an ID at the end of the URI or
                // not. If we do, pass that into the service as the ID to 
                // work with (else default to 0, since the database starts
                // identities at 1).
                string candidateID = tbServiceUri.Text.Substring(tbServiceUri.Text.LastIndexOf('/')+1);
                Int32 blogID = 0;
                Int32.TryParse(candidateID, out blogID);

                // We're executing a PUT or POST, so we need a representation
                // to send to the service. We'll ask for the information
                // before communicating with the server.
                NewBlogEntryWindow newEntryDlg = new NewBlogEntryWindow();

                // If we have a blog ID, set it
                if (blogID > 0)
                {
                    newEntryDlg.BlogID = blogID;
                }

                // Show the dialog
                bool? result = newEntryDlg.ShowDialog();
                if (result == true)
                {
                    // Pull blog entry/comment item... Note one will
                    // be completed and one will be null.
                    newEntryItem = newEntryDlg.EntryItem;
                    newCommentItem = newEntryDlg.CommentItem;
                }
                else
                {
                    // User canceled, so exit method
                    return;
                }
            }

            Int32 attempts = 3;
            while (attempts > 0)
            {
                try
                {
                    // Check for authentication
                    string authorizedHeader = String.Empty;
                    if (!String.IsNullOrEmpty(this._username) && !String.IsNullOrEmpty(this._password))
                    {
                        // Create the header
                        string decoded = String.Format("{0}:{1}", this._username, this._password);
                        string encoded = Convert.ToBase64String(Encoding.ASCII.GetBytes(decoded));
                        authorizedHeader = String.Format("Authorization: BASIC {0}", encoded);
                    }

                    // Create the request
                    Uri uri = new Uri(tbServiceUri.Text);
                    HttpWebRequest request = WebRequest.Create(uri) as HttpWebRequest;
                    request.Method = ((ComboBoxItem)cmbMethod.Items[cmbMethod.SelectedIndex]).Tag.ToString();
                    request.Accept = ((ComboBoxItem)cmbContentType.Items[cmbContentType.SelectedIndex]).Tag.ToString();
                    if (!String.IsNullOrEmpty(authorizedHeader))
                    {
                        request.Headers.Add(authorizedHeader);
                    }

                    // Send any data
                    switch (cmbMethod.SelectedIndex)
                    {
                        case 0: // HEAD, no data
                        case 1: // GET, no data
                        case 4: // DELETE, no data
                            // No data to send, just ask for response
                            break;

                        case 2: // PUT
                        case 3: // POST
                            switch (cmbContentType.SelectedIndex)
                            {
                                case 0: // XML
                                    // Determine the type
                                    if (newEntryItem != null)
                                    {
                                        // Blog entry
                                        PushEntryItemXml(request, newEntryItem);
                                    }
                                    else
                                    {
                                        // Blog comment
                                        PushCommentItemXml(request, newCommentItem);
                                    }
                                    break;

                                case 1: // JSON
                                    // Determine the type
                                    if (newEntryItem != null)
                                    {
                                        // Blog entry
                                        PushEntryItemJson(request, newEntryItem);
                                    }
                                    else
                                    {
                                        // Blog comment
                                        PushCommentItemJson(request, newCommentItem);
                                    }
                                    break;
                            }
                            break;
                    }

                    // Call for the response
                    HttpWebResponse response = request.GetResponse() as HttpWebResponse;

                    // If we made a HEAD call, display the headers, else the content
                    if (cmbMethod.SelectedIndex == 0)
                    {
                        // Build response header output
                        StringBuilder sb = new StringBuilder();
                        sb.AppendFormat("HTTP/{0} {1} {2}\r\n", response.ProtocolVersion, (Int32)response.StatusCode, response.StatusDescription);

                        // Retrieve the response packet headers
                        for (Int32 i = response.Headers.Count - 1; i >= 0; i--)
                        {
                            sb.AppendFormat("{0}: {1}\r\n", response.Headers.GetKey(i), response.Headers[i]);
                        }

                        // Assign text
                        tbResults.Text = sb.ToString();
                    }
                    else
                    {
                        // Retrieve content
                        string responseContent = String.Empty;
                        using (StreamReader rdr = new StreamReader(response.GetResponseStream()))
                        {
                            responseContent = rdr.ReadToEnd();
                        }

                        tbResults.Text = responseContent;
                    }

                    // Clear username/password if we're to forget
                    if (!this._rememberMe)
                    {
                        this._username = String.Empty;
                        this._password = String.Empty;
                    }

                    // Exit the loop
                    break;
                }
                catch (WebException ex)
                {
                    // Check for authentication
                    HttpWebResponse response = ex.Response as HttpWebResponse;
                    if (response.StatusCode == HttpStatusCode.Unauthorized)
                    {
                        // Pop up new username/password dialog box
                        ClientAuthenticationWindow authWindow = new ClientAuthenticationWindow();
                        bool? result = authWindow.ShowDialog();
                        if (result == true)
                        {
                            // Save username/password...
                            this._username = authWindow.Username;
                            this._password = authWindow.Password;

                            // Save "remember me" setting
                            this._rememberMe = authWindow.RememberMeChecked;
                        }
                        else
                        {
                            // User canceled...
                            break;
                        }
                    }
                    else
                    {
                        // Pop up a messagebox showing the error...
                        MessageBox.Show(ex.Message, "Error Invoking RESTful Service", MessageBoxButton.OK, MessageBoxImage.Error);
                        tbResults.Text = ex.ToString();
                        break;
                    }
                }

                // Decrease attempt number
                --attempts;
            }
        }

        private void Exit_Click(object sender, RoutedEventArgs e)
        {
            // Exit the app
            Properties.Settings.Default.Save();
            this.Close();
        }

        private void PushEntryItemXml(HttpWebRequest request, BlogEntryItem entryItem)
        {
            Stream requestStream = request.GetRequestStream();
            DataContractSerializer blogSerializer =
                new DataContractSerializer(typeof(BlogEntryItem));
            blogSerializer.WriteObject(
                requestStream, entryItem);

            // Clear the stream;
            requestStream.Flush();
            requestStream.Close();

            // Set the content type
            request.ContentType = "text/xml";
        }

        private void PushEntryItemJson(HttpWebRequest request, BlogEntryItem entryItem)
        {
            Stream requestStream = request.GetRequestStream();
            DataContractJsonSerializer blogSerializer =
                new DataContractJsonSerializer(typeof(BlogEntryItem));
            blogSerializer.WriteObject(
                requestStream, entryItem);

            // Clear the stream;
            requestStream.Flush();
            requestStream.Close();

            // Set the content type
            request.ContentType = "application/json";
        }

        private void PushCommentItemXml(HttpWebRequest request, BlogCommentItem commentItem)
        {
            Stream requestStream = request.GetRequestStream();
            DataContractSerializer blogSerializer =
                new DataContractSerializer(typeof(BlogCommentItem));
            blogSerializer.WriteObject(
                requestStream, commentItem);

            // Clear the stream;
            requestStream.Flush();
            requestStream.Close();

            // Set the content type
            request.ContentType = "text/xml";
        }

        private void PushCommentItemJson(HttpWebRequest request, BlogCommentItem commentItem)
        {
            Stream requestStream = request.GetRequestStream();
            DataContractJsonSerializer blogSerializer =
                new DataContractJsonSerializer(typeof(BlogCommentItem));
            blogSerializer.WriteObject(
                requestStream, commentItem);

            // Clear the stream;
            requestStream.Flush();
            requestStream.Close();

            // Set the content type
            request.ContentType = "application/json";
        }
    }
}
