﻿using System;
using System.Net;
using System.Configuration;
using System.Security;
using System.Security.Principal;
using System.Web;
using System.Web.Configuration;
using System.Web.Security;
using System.Text;

using AspNetMvcRestService.Security;

namespace AspNetMvcRestService
{
    #region AuthorizationManager

    internal sealed class AuthorizationManager
    {
        private CommaDelimitedStringCollection _allowedRoles = new CommaDelimitedStringCollection();
        private CommaDelimitedStringCollection _deniedRoles = new CommaDelimitedStringCollection();
        private CommaDelimitedStringCollection _allowedUsers = new CommaDelimitedStringCollection();
        private CommaDelimitedStringCollection _deniedUsers = new CommaDelimitedStringCollection();

        private AuthorizationManager()
        {
        }

        public CommaDelimitedStringCollection AllowedRoles
        {
            get { return _allowedRoles; }
        }

        public CommaDelimitedStringCollection DeniedRoles
        {
            get { return _deniedRoles; }
        }

        public CommaDelimitedStringCollection AllowedUsers
        {
            get { return _allowedUsers; }
        }

        public CommaDelimitedStringCollection DeniedUsers
        {
            get { return _deniedUsers; }
        }

        public bool HasRoles
        {
            get { return (AllowedRoles.Count > 0 || DeniedRoles.Count > 0); }
        }

        public bool HasNonDefaultUsers
        {
            get
            {
                // If there is only the default user indicated,
                // ignore that special case and return the
                // fact we have no other users than the
                // default.
                bool hasUsers = false;
                if (AllowedUsers.Count == 1 &&
                    AllowedUsers.Contains("*") &&
                    AllowedRoles.Count == 0)
                {
                    // Do nothing
                    ;
                }
                else
                {
                    // Check if we have allowed users
                    hasUsers |= AllowedUsers.Count > 0;

                    // Check if we have denied users
                    hasUsers |= DeniedUsers.Count > 0;
                }

                return hasUsers;
            }
        }

        public bool RequiresAuthorization
        {
            get { return (HasRoles || HasNonDefaultUsers); }
        }

        public bool VerifyAuthorization(IPrincipal user)
        {
            // check user against a set of business rules:
            //
            // 1) If the user is denied by username, deny the user
            // 2) If the user is allowed by default without roles, allow user
            // 3) If the user is denied by default without roles, deny user
            // 4) If the user is denied by default but allowed based on role
            //    membership, allow user
            bool userDeniedByName = DeniedUsers.Contains(user.Identity.Name);
            bool userAllowedByDefault = ((AllowedUsers.Contains(user.Identity.Name) || AllowedUsers.Contains("*")) && AllowedRoles.Count == 0);
            bool userDeniedByDefault = ((DeniedUsers.Contains("*") || AllowedUsers.Contains("?")) && AllowedRoles.Count == 0);
            bool userAllowedByRole = false;
            foreach (string role in AllowedRoles)
            {
                if (user.IsInRole(role))
                {
                    userAllowedByRole = true;
                    break;
                }
            }

            return (userDeniedByName || userAllowedByDefault || userDeniedByDefault || userAllowedByRole);
        }

        public static AuthorizationManager CreateAuthorizationManager(string url)
        {
            Configuration config =
            WebConfigurationManager.OpenWebConfiguration(url);
            AuthorizationSection configSection =
            (AuthorizationSection)config.GetSection("system.web/authorization");
            AuthorizationRuleCollection rules = configSection.Rules;

            AuthorizationManager configInfo = new AuthorizationManager();
            for (int i = 0; i < rules.Count; i++)
            {
                if (rules[i].Roles.Count > 0)
                {
                    if (rules[i].Action.ToString() == "Allow")
                        configInfo.AllowedRoles.AddRange(rules[i].Roles.ToString().Split(','));
                    else if (rules[i].Action.ToString() == "Deny")
                        configInfo.DeniedRoles.AddRange(rules[i].Roles.ToString().Split(','));
                }
                if (rules[i].Users.Count > 0)
                {
                    if (rules[i].Action.ToString() == "Allow")
                        configInfo.AllowedUsers.AddRange(rules[i].Users.ToString().Split(','));
                    else if (rules[i].Action.ToString() == "Deny")
                        configInfo.DeniedUsers.AddRange(rules[i].Users.ToString().Split(','));
                }
            }

            return configInfo;
        }
    }

    #endregion

    public class AuthModule : IHttpModule
    {
        const string BasicAuthProtocol = "basic";

        /// <summary>
        /// You will need to configure this module in the web.config file of your
        /// web and register it with IIS before being able to use it. For more information
        /// see the following link: http://go.microsoft.com/?linkid=8101007
        /// 
        /// In addition, this module takes over the functions of the default
        /// UrlAuthorization module, so you'll need to remove it:
        /// 
        /// &lt;remove name="UrlAuthorization" /&gt;
        /// 
        /// For more information see the following link:
        /// http://technet.microsoft.com/en-us/library/cc731223.aspx
        /// </summary>
        #region IHttpModule Members

        public void Dispose()
        {
        }

        public void Init(HttpApplication context)
        {
            // Hook the authentication request event
            context.AuthenticateRequest += new EventHandler(OnAuthenticateRequest);
        }

        #endregion

        #region Event handlers

        void OnAuthenticateRequest(object sender, EventArgs e)
        {
            // Pull the current context
            HttpContext context = HttpContext.Current;

            // Pull the authorization information for this URI
            string url = context.Request.Url.LocalPath;
            AuthorizationManager authMgr = AuthorizationManager.CreateAuthorizationManager(url);

            // Check to see if we need authentication
            if (authMgr.RequiresAuthorization)
            {
                // Look for the authorization header
                bool isAuthorized = false;
                string authHeader = context.Request.Headers["Authorization"];
                if (!String.IsNullOrEmpty(authHeader) && authHeader.ToLower().StartsWith(BasicAuthProtocol))
                {
                    // Pull the credentials
                    string[] credentials = stripCredentials(authHeader);

                    // Authenticate
                    if (credentials.Length == 2 &&
                        Membership.ValidateUser(credentials[0], credentials[1]))
                    {
                        // Set the caller's credentials...note we may rescind this
                        // depending upon the authorization check from the configuration
                        // file.
                        context.User = new ServiceUser(credentials[0], true);

                        // Verify the configuration file supports this user
                        if (authMgr.VerifyAuthorization(context.User))
                        {
                            // Mark as authorized
                            isAuthorized = true;
                        }
                        else
                        {
                            // Rescind temporary authorization and indicate that
                            // the resource is forbidden.
                            context.Response.StatusCode = (Int32)HttpStatusCode.Forbidden;

                            // End the request
                            context.Response.End();
                            return;
                        }
                    }
                }

                // Check for authentication and return HTTP 401 if none
                if (!isAuthorized)
                {
                    // Assign the status code
                    context.Response.StatusCode = (Int32)HttpStatusCode.Unauthorized;

                    // Calculate the realm
                    string realm = String.Format("Basic realm=\"{0}\"", context.Request.Url.GetLeftPart(UriPartial.Path));

                    // Add the authentication header
                    context.Response.AddHeader("WWW-Authenticate", realm);

                    // End the request
                    context.Response.End();
                }
            }
        }

        #endregion

        #region Helpers

        string[] stripCredentials(string authHeader)
        {
            // Remove the "basic"
            string rawCredentials = authHeader.Substring(6).Trim();

            // Remove Base64 encoding
            byte[] bytes = Convert.FromBase64String(rawCredentials);
            string credentials = UTF8Encoding.UTF8.GetString(bytes);

            // Return separated string array
            return credentials.Split(':');
        }

        #endregion
    }
}
