﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Transactions;
using System.IO;

namespace AspNetMvcRestService.Models
{
    public sealed class CodeXRCDataAccess
    {
        CodeXRCAuxServices _aux = new CodeXRCAuxServices();

        #region GET methods

        public List<ProjectDTO> ListProjects(ControllerContext context, Guid ownerID)
        {
            // Retrieve all projects
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var projectQuery = from p in dataContext.Projects orderby p.ProjectID where p.OwnerID == ownerID select p;
            List<ProjectDTO> projects = new List<ProjectDTO>();
            foreach (var project in projectQuery)
            {
                // Create the new project item
                ProjectDTO projectItem = new ProjectDTO(project);
                projectItem.Link = CodeXRCLinkBuilder.BuildProjectLink(context, project.DisplayName);
                projects.Add(projectItem);
            }
            return projects;
        }

        public List<FolderDTO> ListFolders(ControllerContext context, Guid ownerID, string projectName)
        {
            // Retrieve all folders for project
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var projectQuery = from p in dataContext.Projects where p.OwnerID == ownerID && p.DisplayName.ToLower() == projectName.ToLower() select p;
            if (projectQuery.Count() == 0) return null;
            var folderQuery = from f in dataContext.Folders orderby f.FolderID where f.ProjectID == projectQuery.First().ProjectID select f;
            List<FolderDTO> folders = new List<FolderDTO>();
            foreach (var folder in folderQuery)
            {
                // Create the new folder item
                FolderDTO folderItem = new FolderDTO(folder);
                folderItem.Link = CodeXRCLinkBuilder.BuildFolderLink(context, projectName, folder.DisplayName);
                folders.Add(folderItem);

                // Add any child folders
                List<FolderDTO> childFolders = ListChildFolders(context, ownerID, folder.FolderID, projectName);
                if (childFolders != null && childFolders.Count() > 0)
                {
                    folderItem.Folders = childFolders.ToArray();
                }

                // Add any subordinate files
                List<FileDTO> files = ListFiles(context, ownerID, folder.FolderID, projectName, folder.DisplayName);
                if (files != null)
                {
                    folderItem.Files = files.ToArray();
                }
            }
            return folders;
        }

        public List<FolderDTO> ListFoldersByFolder(ControllerContext context, Guid ownerID, string projectName, string folderName)
        {
            // Retrieve all folders
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var folderQuery = from f in dataContext.Folders orderby f.FolderID where f.DisplayName.ToLower() == folderName.ToLower() select f;
            if (folderQuery.Count() == 0) return null;
            var childFolderQuery = from f in dataContext.Folders orderby f.FolderID where f.ParentFolderID == folderQuery.First().FolderID select f;
            if (childFolderQuery.Count() == 0) return null;
            List<FolderDTO> folders = new List<FolderDTO>();
            foreach (var folder in childFolderQuery)
            {
                // Create the new folder item
                FolderDTO folderItem = new FolderDTO(folder);
                folderItem.Link = CodeXRCLinkBuilder.BuildFolderLink(context, projectName, folder.DisplayName);
                folders.Add(folderItem);

                // Add any child folders
                List<FolderDTO> childFolders = ListChildFolders(context, ownerID, folder.FolderID, projectName);
                if (childFolders != null && childFolders.Count() > 0)
                {
                    folderItem.Folders = childFolders.ToArray();
                }

                // Add any subordinate files
                List<FileDTO> files = ListFiles(context, ownerID, folder.FolderID, projectName, folder.DisplayName);
                if (files != null)
                {
                    folderItem.Files = files.ToArray();
                }
            }
            return folders;
        }

        public FolderDTO GetFolder(ControllerContext context, Guid ownerID, string projectName, string folderName)
        {
            // Retrieve folder
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var folderQuery = from f in dataContext.Folders orderby f.FolderID where f.DisplayName.ToLower() == folderName.ToLower() select f;
            if (folderQuery.Count() == 0) return null;
            FolderDTO folder = new FolderDTO();

            // Create the new folder item
            FolderDTO folderItem = new FolderDTO(folderQuery.First());
            folderItem.Link = CodeXRCLinkBuilder.BuildFolderLink(context, projectName, folder.DisplayName);

            return folderItem;
        }

        public List<FolderDTO> ListChildFolders(ControllerContext context, Guid ownerID, Int32 parentFolderID, string projectName)
        {
            // Retrieve all folders
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var folderQuery = from f in dataContext.Folders orderby f.FolderID where f.ParentFolderID == parentFolderID select f;
            if (folderQuery.Count() == 0) return null;
            List<FolderDTO> folders = new List<FolderDTO>();
            foreach (var folder in folderQuery)
            {
                // Create the new folder item
                FolderDTO folderItem = new FolderDTO(folder);
                folderItem.Link = CodeXRCLinkBuilder.BuildFolderLink(context, projectName, folder.DisplayName);
                folders.Add(folderItem);

                // Add any child folders
                List<FolderDTO> childFolders = ListChildFolders(context, ownerID, folder.FolderID, projectName);
                if (childFolders != null && childFolders.Count() > 0)
                {
                    folderItem.Folders = childFolders.ToArray();
                }

                // Add any subordinate files
                List<FileDTO> files = ListFiles(context, ownerID, folder.FolderID, projectName, folder.DisplayName);
                folderItem.Files = files.ToArray();
            }
            return folders;
        }

        public List<FileDTO> ListFiles(ControllerContext context, Guid ownerID, Int32 folderID, string projectName, string folderName)
        {
            // Retrieve all files
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var fileQuery = from f in dataContext.Files orderby f.FileID where f.FolderID == folderID select f;
            List<FileDTO> files = new List<FileDTO>();
            foreach (var file in fileQuery)
            {
                // Create the new file item
                FileDTO fileItem = new FileDTO(file);
                fileItem.Link = CodeXRCLinkBuilder.BuildFileLink(context, projectName, folderName, file.Extension, file.DisplayName);
                files.Add(fileItem);
            }
            return files;
        }

        public List<FileDTO> ListFiles(ControllerContext context, Guid ownerID, string projectName, string folderName)
        {
            // Retrieve all files
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var folderQuery = from f in dataContext.Folders orderby f.FolderID where f.DisplayName.ToLower() == folderName.ToLower() select f;
            if (folderQuery.Count() == 0) return null;
            var fileQuery = from f in dataContext.Files orderby f.FileID where f.FolderID == folderQuery.First().FolderID select f;
            List<FileDTO> files = new List<FileDTO>();
            foreach (var file in fileQuery)
            {
                // Create the new file item
                FileDTO fileItem = new FileDTO(file);
                fileItem.Link = CodeXRCLinkBuilder.BuildFileLink(context, projectName, folderName, file.Extension, file.DisplayName);
                files.Add(fileItem);
            }
            return files;
        }

        public FileDTO[] GetFileByType(ControllerContext context, Guid ownerID, string projectName, string folderName, string fileExtension)
        {
            // Retrieve none to many files
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var folderQuery = from f in dataContext.Folders orderby f.FolderID where f.DisplayName.ToLower() == folderName.ToLower() select f;
            if (folderQuery.Count() == 0) return null;
            var fileQuery = from f in dataContext.Files orderby f.FileID where f.Extension.ToLower() == fileExtension.ToLower() && f.FolderID == folderQuery.First().FolderID select f;
            List<FileDTO> files = new List<FileDTO>();
            foreach (var file in fileQuery)
            {
                FileDTO fileItem = new FileDTO(file);
                fileItem.Link = CodeXRCLinkBuilder.BuildFolderLink(context, projectName, folderName);
                files.Add(fileItem);
            }
            return files.ToArray();
        }

        public FileDTO GetFile(ControllerContext context, Guid ownerID, string projectName, string folderName, string fileName)
        {
            // Retrieve one file
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var folderQuery = from f in dataContext.Folders orderby f.FolderID where f.DisplayName.ToLower() == folderName.ToLower() select f;
            if (folderQuery.Count() == 0) return null;
            var fileQuery = from f in dataContext.Files orderby f.FileID where f.DisplayName == fileName && f.FolderID == folderQuery.First().FolderID select f;
            FileDTO file = null;
            if (fileQuery.Count() == 1)
            {
                file = new FileDTO(fileQuery.First());
                file.Contents = _aux.RenderBinaryContent(context, file.LocalStorage);
                file.Link = CodeXRCLinkBuilder.BuildFolderLink(context, projectName, folderName);
            }
            return file;
        }

        #endregion

        #region PUT methods

        public void CreateOrUpdateProject(ControllerContext context, Guid ownerID, ProjectDTO project)
        {
            // Query for the project
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var projectQuery = from p in dataContext.Projects where p.DisplayName.ToLower() == project.DisplayName.ToLower() && p.OwnerID == ownerID select p;
            try
            {
                if (projectQuery.Count() == 0)
                {
                    // Insert... Note since the display names are the same,
                    // we have no need to update.
                    Project dbProject = new Project();
                    dbProject.DisplayName = project.DisplayName;
                    dbProject.OwnerID = ownerID;
                    dbProject.LocalStorage = context.HttpContext.Server.MapPath("~/App_Data") + "\\" + project.DisplayName;
                    dataContext.Projects.InsertOnSubmit(dbProject);

                    // Create the directory
                    Directory.CreateDirectory(dbProject.LocalStorage);

                    // Transact the insert/update in the database
                    using (TransactionScope ts = new TransactionScope())
                    {
                        // Submit the entry
                        dataContext.SubmitChanges();

                        // Commit the insert
                        ts.Complete();
                    }

                    // Assign the project's ID
                    project.ProjectID = dbProject.ProjectID;
                }
            }
            catch (Exception ex)
            {
                // Issues creating new project...
                _aux.RenderErrorInternalError(context, String.Format("Error recording new project resource: {0}.", ex.Message));
                throw new CodeXRCException();
            }
        }

        public void CreateOrUpdateProjectFolder(ControllerContext context, Guid ownerID, string projectName, FolderDTO folder)
        {
            // Query for the project
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var projectQuery = from p in dataContext.Projects where p.DisplayName.ToLower() == projectName.ToLower() && p.OwnerID == ownerID select p;
            if (projectQuery.Count() > 0)
            {
                // Delegate
                CreateOrUpdateProjectFolder(context, ownerID, projectQuery.First().ProjectID, projectName, folder);
            }
        }

        public void CreateOrUpdateProjectFolder(ControllerContext context, Guid ownerID, Int32 projectID, string projectName, FolderDTO folder)
        {
            // Query for the folder
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var folderQuery = from f in dataContext.Folders where f.DisplayName.ToLower() == folder.DisplayName.ToLower() && f.ProjectID == projectID select f;
            try
            {
                if (folderQuery.Count() == 0)
                {
                    // Insert... Note since the display names are the same,
                    // we have no need to update.
                    Folder dbFolder = new Folder();
                    dbFolder.DisplayName = folder.DisplayName;
                    dbFolder.OwnerID = ownerID;
                    dbFolder.ProjectID = projectID;
                    dbFolder.ParentFolderID = -1;
                    dbFolder.LocalStorage = context.HttpContext.Server.MapPath("~/App_Data") + "\\" + projectName + "\\" + dbFolder.DisplayName;
                    dataContext.Folders.InsertOnSubmit(dbFolder);

                    // Create the directory
                    Directory.CreateDirectory(dbFolder.LocalStorage);

                    // Transact the insert/update in the database
                    using (TransactionScope ts = new TransactionScope())
                    {
                        // Submit the entry
                        dataContext.SubmitChanges();

                        // Commit the insert
                        ts.Complete();
                    }

                    // Assign the folder's ID information
                    folder.FolderID = dbFolder.FolderID;
                    folder.ParentFolderID = -1;
                    folder.ProjectID = dbFolder.ProjectID;
                }
            }
            catch (Exception ex)
            {
                // Issues saving...
                _aux.RenderErrorInternalError(context, String.Format("Error recording new folder resource: {0}.", ex.Message));
                throw new CodeXRCException();
            }
        }

        public void CreateOrUpdateChildFolder(ControllerContext context, Guid ownerID, Int32 projectID, string projectName, Int32 parentFolderID, FolderDTO folder)
        {
            // Query for the folder
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var folderQuery = from f in dataContext.Folders where f.DisplayName.ToLower() == folder.DisplayName.ToLower() && f.ParentFolderID == parentFolderID select f;
            var parentQuery = from f in dataContext.Folders where f.FolderID == parentFolderID select f; // should exist...
            try
            {
                if (folderQuery.Count() == 0)
                {
                    // Insert... Note since the display names are the same,
                    // we have no need to update.
                    Folder dbFolder = new Folder();
                    dbFolder.DisplayName = folder.DisplayName;
                    dbFolder.OwnerID = ownerID;
                    dbFolder.ProjectID = projectID;
                    dbFolder.ParentFolderID = parentFolderID;
                    dbFolder.LocalStorage = parentQuery.First().LocalStorage + "\\" + dbFolder.DisplayName;
                    dataContext.Folders.InsertOnSubmit(dbFolder);

                    // Create the directory
                    Directory.CreateDirectory(dbFolder.LocalStorage);

                    // Transact the insert/update in the database
                    using (TransactionScope ts = new TransactionScope())
                    {
                        // Submit the entry
                        dataContext.SubmitChanges();

                        // Commit the insert
                        ts.Complete();
                    }

                    // Assign the folder's ID information
                    folder.FolderID = dbFolder.FolderID;
                    folder.ParentFolderID = dbFolder.ParentFolderID;
                    folder.ProjectID = dbFolder.ProjectID;
                }
            }
            catch (Exception ex)
            {
                // Issues deserializing...
                _aux.RenderErrorInternalError(context, String.Format("Error recording new folder resource: {0}.", ex.Message));
                throw new CodeXRCException();
            }
        }

        public void CreateOrUpdateFile(ControllerContext context, Guid ownerID, string projectName, Int32 parentFolderID, FileDTO file)
        {
            // Query for the project
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var projectQuery = from p in dataContext.Projects where p.DisplayName.ToLower() == projectName.ToLower() && p.OwnerID == ownerID select p;
            if (projectQuery.Count() == 0) return;

            // Query for the folder
            var folderQuery = from f in dataContext.Folders where f.FolderID == parentFolderID select f;
            if (folderQuery.Count() == 0) return;

            // Query for the file
            var fileQuery = from f in dataContext.Files where f.DisplayName.ToLower() == file.DisplayName.ToLower() && f.FolderID == parentFolderID select f;
            try
            {
                File dbFile = null;
                if (fileQuery.Count() == 0)
                {
                    // Insert
                    dbFile = new File();
                    dbFile.DisplayName = file.DisplayName;
                    dbFile.FolderID = parentFolderID;
                    dbFile.ProjectID = projectQuery.First().ProjectID;
                    dbFile.OwnerID = ownerID;
                    dbFile.Extension = file.DisplayName.LastIndexOf('.') != -1 ? file.DisplayName.Substring(file.DisplayName.LastIndexOf('.')+1) : String.Empty;
                    dbFile.LocalStorage = folderQuery.First().LocalStorage + "\\" + file.DisplayName;
                    dataContext.Files.InsertOnSubmit(dbFile);
                }
                else
                {
                    // Return existing file
                    dbFile = fileQuery.First();
                }

                // Create the file
                using (FileStream fstm = new FileStream(dbFile.LocalStorage, FileMode.OpenOrCreate, FileAccess.Write, FileShare.None))
                {
                    // Pull the contents as Base64
                    byte[] contents = Convert.FromBase64String(file.Contents);

                    // Write as stream
                    MemoryStream mstm = new MemoryStream(contents);

                    // Copy contents and close
                    _aux.CopyStream(mstm, fstm);
                    fstm.Flush();
                    fstm.Close();
                }

                // Transact the insert/update in the database
                using (TransactionScope ts = new TransactionScope())
                {
                    // Submit the entry
                    dataContext.SubmitChanges();

                    // Commit the insert
                    ts.Complete();
                }

                // Assign the file's ID information
                file.FileID = dbFile.FileID;
                file.FolderID = dbFile.FolderID;
                file.ProjectID = dbFile.ProjectID;
            }
            catch (Exception ex)
            {
                // Issues deserializing...
                _aux.RenderErrorInternalError(context, String.Format("Error recording new file resource: {0}.", ex.Message));
                throw new CodeXRCException();
            }
        }

        #endregion

        #region DELETE methods

        public void DeleteProjectsForUser(ControllerContext context, Guid ownerID)
        {
            // Delete all project records and files for user
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var projectQuery = from p in dataContext.Projects where p.OwnerID == ownerID select p;

            // Delete the project. Note if it doesn't exist, we won't
            // return any sort of error. We'll just return. From
            // a data management perspective, we might prefer to set
            // a bit in the record that says "deleted," but for 
            // demonstration purposes I'll actually delete the
            // project record and any related folders/files.
            if (projectQuery.Count() > 0)
            {
                foreach (Project project in projectQuery)
                {
                    try
                    {
                        // Delete the underlying directory in the file system
                        System.IO.Directory.Delete(project.LocalStorage, true);
                    }
                    catch
                    {
                        // Ignore exceptions so we don't lose the
                        // database transaction...
                    }
                }

                // Cascaded delete
                dataContext.Projects.DeleteAllOnSubmit(projectQuery);
            }

            // Transact the delete in the database. I could have
            // called other data access methods to delete folders
            // and files, but I wanted the delete transacted in
            // a single transaction scope.
            using (TransactionScope ts = new TransactionScope())
            {
                // Submit the entry
                dataContext.SubmitChanges();

                // Commit the insert
                ts.Complete();
            }
        }

        public void DeleteProject(ControllerContext context, Guid ownerID, string projectName)
        {
            // Delete this project
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var projectQuery = from p in dataContext.Projects where p.DisplayName.ToLower() == projectName.ToLower() && p.OwnerID == ownerID select p;
            if (projectQuery.Count() > 0)
            {
                try
                {
                    Directory.Delete(projectQuery.First().LocalStorage, true);
                }
                catch
                {
                    // Ignore exceptions so we don't lose the
                    // database transaction...
                }

                // Cascaded delete
                dataContext.Projects.DeleteOnSubmit(projectQuery.First());

                // Transact the delete in the database
                using (TransactionScope ts = new TransactionScope())
                {
                    // Submit the entry
                    dataContext.SubmitChanges();

                    // Commit the insert
                    ts.Complete();
                }
            }
        }

        public void DeleteFolder(ControllerContext context, Guid ownerID, string projectName, string folderName)
        {
            // Delete the files
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var projectQuery = from p in dataContext.Projects where p.OwnerID == ownerID select p;
            if (projectQuery.Count() == 0) return;
            var folderQuery = from f in dataContext.Folders where f.DisplayName.ToLower() == folderName.ToLower() && f.ProjectID == projectQuery.First().ProjectID select f;

            // Delete the folder and all child folders
            if (folderQuery.Count() > 0)
            {
                try
                {
                    Directory.Delete(folderQuery.First().LocalStorage, true);
                }
                catch
                {
                    // Ignore exceptions so we don't lose the
                    // database transaction...
                }

                // Cascaded delete
                dataContext.Folders.DeleteOnSubmit(folderQuery.First());

                // Transact the delete in the database
                using (TransactionScope ts = new TransactionScope())
                {
                    // Submit the entry
                    dataContext.SubmitChanges();

                    // Commit the insert
                    ts.Complete();
                }
            }
        }

        public void DeleteFileByType(ControllerContext context, Guid ownerID, string projectName, string folderName, string fileExtension)
        {
            // Delete this type of file in the specified folder
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var projectQuery = from p in dataContext.Projects where p.DisplayName.ToLower() == projectName.ToLower() && p.OwnerID == ownerID select p;
            if (projectQuery.Count() == 0) return;
            var folderQuery = from f in dataContext.Folders where f.DisplayName.ToLower() == folderName.ToLower() && f.ProjectID == projectQuery.First().ProjectID select f;

            if (folderQuery.Count() > 0)
            {
                // Delete the files
                var fileQuery = from f in dataContext.Files where f.FolderID == folderQuery.First().FolderID && f.Extension.ToLower() == fileExtension.ToLower() select f;
                if (fileQuery.Count() > 0)
                {
                    foreach (File file in fileQuery)
                    {
                        try
                        {
                            // Delete the underlying file in the file system
                            System.IO.File.Delete(file.LocalStorage);
                        }
                        catch
                        {
                            // Ignore exceptions so we don't lose the
                            // database transaction...
                        }
                    }
                    dataContext.Files.DeleteAllOnSubmit(fileQuery);
                }

                // Transact the delete in the database
                using (TransactionScope ts = new TransactionScope())
                {
                    // Submit the entry
                    dataContext.SubmitChanges();

                    // Commit the insert
                    ts.Complete();
                }
            }
        }

        public void DeleteFileByType(ControllerContext context, Guid ownerID, CodeXRCDataClassesDataContext dataContext, Int32 parentFolderID, string folderName, string fileExtension)
        {
            var folderQuery = from f in dataContext.Folders where f.ParentFolderID == parentFolderID select f;
            if (folderQuery.Count() > 0)
            {
                foreach (Folder folder in folderQuery)
                {
                    // Check if this is the folder
                    if (folderName.ToLower() == folder.DisplayName.ToLower())
                    {
                        // Delete the files
                        var fileQuery = from f in dataContext.Files where f.FolderID == folder.FolderID && f.Extension.ToLower() == fileExtension.ToLower() select f;
                        if (fileQuery.Count() > 0)
                        {
                            foreach (File file in fileQuery)
                            {
                                try
                                {
                                    // Delete the underlying file in the file system
                                    System.IO.File.Delete(file.LocalStorage);
                                }
                                catch
                                {
                                    // Ignore exceptions so we don't lose the
                                    // database transaction...
                                }
                            }
                            dataContext.Files.DeleteAllOnSubmit(fileQuery);
                        }

                        // Done
                        return;
                    }
                    else
                    {
                        // Recurse
                        DeleteFileByType(context, ownerID, dataContext, folder.FolderID, folderName, fileExtension);
                    }
                }
            }
        }


        public void DeleteFile(ControllerContext context, Guid ownerID, string projectName, string folderName, string fileName)
        {
            // Delete this file
            CodeXRCDataClassesDataContext dataContext = new CodeXRCDataClassesDataContext();
            var projectQuery = from p in dataContext.Projects where p.DisplayName.ToLower() == projectName.ToLower() && p.OwnerID == ownerID select p;
            if (projectQuery.Count() == 0) return;
            var folderQuery = from f in dataContext.Folders where f.DisplayName.ToLower() == folderName.ToLower() && f.ProjectID == projectQuery.First().ProjectID select f;
            if (folderQuery.Count() == 0) return;

            if (folderQuery.Count() > 0)
            {
                // Delete the file
                var fileQuery = from f in dataContext.Files where f.FolderID == folderQuery.First().FolderID && f.DisplayName.ToLower() == fileName.ToLower() select f;
                if (fileQuery.Count() > 0)
                {
                    try
                    {
                        // Delete the underlying file in the file system
                        System.IO.File.Delete(fileQuery.First().LocalStorage);
                    }
                    catch
                    {
                        // Ignore exceptions so we don't lose the
                        // database transaction...
                    }
                    dataContext.Files.DeleteOnSubmit(fileQuery.First());
                }

                // Transact the delete in the database
                using (TransactionScope ts = new TransactionScope())
                {
                    // Submit the entry
                    dataContext.SubmitChanges();

                    // Commit the insert
                    ts.Complete();
                }
            }
        }

        #endregion
    }
}
