﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.Mvc;
using System.Web.Security;
using System.IO;
using System.Text;
using System.Runtime.Serialization;

using AspNetMvcRestService.Security;

namespace AspNetMvcRestService.Models
{
    public partial class CodeXRCPutService
    {
        private void CreateResource(ControllerContext context)
        {
            // Pull the user's ID. This works here since we're using the SQL-based
            // membership provider.
            MembershipUser user = Membership.GetUser(context.HttpContext.User.Identity.Name);
            Guid ownerID = (Guid)user.ProviderUserKey;

            context.HttpContext.Response.ContentType = "text/html";
            switch (_directives.Count())
            {
                case 0:
                default:
                    // Create a project for user (will be a specific project):
                    // http://{host}/{vdir}/CodeXRC
                    CreateProject(context, ownerID);
                    break;

                case 1:
                    // Create specific project for user (all child folders and files):
                    // http://{host}/{vdir}/CodeXRC/{project}
                    CreateProject(context, ownerID, _directives[0]);
                    break;

                case 2:
                    // Delete specific folder (all child folders and files):
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}
                    CreateFolder(context, ownerID, _directives[0], _directives[1]);
                    break;

                case 3:
                    // Delete files by extension in specified folder:
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}/{file extension}
                    CreateFileByType(context, ownerID, _directives[0], _directives[1], _directives[2]);
                    break;

                case 4:
                    // Delete specific file:
                    // http://{host}/{vdir}/CodeXRC/{project}/{folder}/{file extension}/{file name}
                    CreateFile(context, ownerID, _directives[0], _directives[1], _directives[3] + "." + _directives[2]);
                    break;
            }
        }

        private void CreateProject(ControllerContext context, Guid ownerID)
        {
            // Delegate to the overloaded method. That method doesn't use the
            // project name anyway. It gets it from the XML project DTO.
            CreateProject(context, ownerID, String.Empty);
        }

        private void CreateProject(ControllerContext context, Guid ownerID, string projectName)
        {
            // Deserialize the XML
            DataContractSerializer serializer = new DataContractSerializer(typeof(ProjectDTO));
            ProjectDTO project = null;
            bool createProject = true;
            try
            {
                // Deserialize
                project = serializer.ReadObject(context.HttpContext.Request.InputStream) as ProjectDTO;

                // Check to see that we have an object
                if (project == null)
                {
                    // Issues deserializing...
                    _aux.RenderErrorInternalError(context, "Error reading XML resource stream: not a valid CodeXRC project XML stream. Please check your XML generation algorithm.");
                    return;
                }

                // Pull the project name, if needed
                if (String.IsNullOrEmpty(projectName))
                {
                    projectName = project.DisplayName;
                }

                // Pull project ID...if -1, we're creating a new
                // project. If not, we're updating one.
                if (project.ProjectID >= 0)
                {
                    // Updating...
                    createProject = false;
                }
            }
            catch
            {
                // Issues deserializing...
                _aux.RenderErrorInternalError(context, "Error reading XML resource stream: could not deserialize the object. Please check your XML generation algorithm.");
                return;
            }

            try
            {
                // Create the project itself
                _dal.CreateOrUpdateProject(context, ownerID, project);

                if (project.Folders != null && project.Folders.Count() > 0)
                {
                    foreach (FolderDTO projectFolder in project.Folders)
                    {
                        // Create or update the top level folder
                        _dal.CreateOrUpdateProjectFolder(context, ownerID, projectName, projectFolder);

                        if (projectFolder.Folders.Count() > 0)
                        {
                            foreach (FolderDTO childFolder in projectFolder.Folders)
                            {
                                // Create or update the child folder
                                childFolder.ParentFolderID = projectFolder.FolderID;
                                childFolder.ProjectID = project.ProjectID;
                                CreateFolderFromFolderDTO(context, ownerID, projectName, childFolder);
                            }
                        }

                        // Create or update any files in the project level folder
                        if (projectFolder.Files.Count() > 0)
                        {
                            foreach (FileDTO file in projectFolder.Files)
                            {
                                // Create or update this file
                                file.FolderID = projectFolder.FolderID;
                                _dal.CreateOrUpdateFile(context, ownerID, project.DisplayName, projectFolder.FolderID, file);

                                // To reduce the size of the returned XML, we'll remove
                                // the file contents.
                                file.Contents = String.Empty;
                           }
                        }
                    }
                }

                // Assign proper return code and tweak headers...
                if (createProject)
                {
                    // Return created/updated project to client
                    serializer.WriteObject(context.HttpContext.Response.OutputStream, project);

                    // Indicate it has been created
                    context.HttpContext.Response.Headers.Add("Location", CodeXRCLinkBuilder.BuildProjectLink(context, project.DisplayName));

                    // Assign created return code
                    _aux.RenderSuccessCreated(context);
                }
                else
                {
                    // Assign OK return code
                    _aux.RenderSuccessOK(context);
                }
            }
            catch (CodeXRCException)
            {
                // Return error already formatted and issued...
            }
            catch (Exception ex)
            {
                // Issues creating new project...
                _aux.RenderErrorInternalError(context, String.Format("Error recording new project resource: {0}.", ex.Message));
            }
        }

        private void CreateFolder(ControllerContext context, Guid ownerID, string projectName, string folderName)
        {
            // Deserialize the XML
            DataContractSerializer serializer = new DataContractSerializer(typeof(FolderDTO));
            FolderDTO folder = null;
            bool createFolder = true;
            try
            {
                // Deserialize
                folder = serializer.ReadObject(context.HttpContext.Request.InputStream) as FolderDTO;

                // Check to see that we have an object
                if (folder == null)
                {
                    // Issues deserializing...
                    _aux.RenderErrorInternalError(context, "Error reading XML resource stream: not a valid CodeXRC folder XML stream. Please check your XML generation algorithm.");
                    return;
                }

                // Pull folder ID...if -1, we're creating a new
                // folder. If not, we're updating one.
                if (folder.FolderID >= 0)
                {
                    // Updating...
                    createFolder = false;
                }
            }
            catch
            {
                // Issues deserializing...
                _aux.RenderErrorInternalError(context, "Error reading XML resource stream: could not deserialize the object. Please check your XML generation algorithm.");
                return;
            } 

            // Create or update...
            CreateFolderFromFolderDTO(context, ownerID, projectName, folder);

            // Assign proper return code and tweak headers...
            if (createFolder)
            {
                // Return created/updated project to client
                serializer.WriteObject(context.HttpContext.Response.OutputStream, folder);

                // Indicate it has been created
                context.HttpContext.Response.Headers.Add("Location", CodeXRCLinkBuilder.BuildFolderLink(context, projectName, folder.DisplayName));

                // Assign created return code
                _aux.RenderSuccessCreated(context);
            }
            else
            {
                // Assign OK return code
                _aux.RenderSuccessOK(context);
            }
        }

        private void CreateFolderFromFolderDTO(ControllerContext context, Guid ownerID, string projectName, FolderDTO folder)
        {
            try
            {
                if (folder.ParentFolderID < 0)
                {
                    // Project level folder...
                    _dal.CreateOrUpdateProjectFolder(context, ownerID, projectName, folder);
                }
                else
                {
                    // Child folder...
                    _dal.CreateOrUpdateChildFolder(context, ownerID, folder.ProjectID, projectName, folder.ParentFolderID, folder);
                }

                if (folder.Folders != null && folder.Folders.Count() > 0)
                {
                    foreach (FolderDTO childFolder in folder.Folders)
                    {
                        // Create or update the child folder
                        CreateFolderFromFolderDTO(context, ownerID, projectName, childFolder);
                    }
                }

                // Create or update any files in this folder
                if (folder.Files.Count() > 0)
                {
                    foreach (FileDTO file in folder.Files)
                    {
                        // Create or update this file
                        _dal.CreateOrUpdateFile(context, ownerID, projectName, folder.FolderID, file);

                        // To reduce the size of the returned XML, we'll remove
                        // the file contents.
                        file.Contents = String.Empty;
                    }
                }
            }
            catch (CodeXRCException)
            {
                // Return error already formatted and issued...
            }
            catch (Exception ex)
            {
                // Issues creating new folder...
                _aux.RenderErrorInternalError(context, String.Format("Error recording new folder resource: {0}.", ex.Message));
            }

        }

        private void CreateFileByType(ControllerContext context, Guid ownerID, string projectName, string folderName, string fileExtension)
        {
            // Deserialize the XML
            DataContractSerializer serializer = new DataContractSerializer(typeof(FileDTO[]));
            FileDTO[] files = null;
            bool createFiles = true;
            try
            {
                // Deserialize
                files = serializer.ReadObject(context.HttpContext.Request.InputStream) as FileDTO[];

                // Check to see that we have an object
                if (files == null)
                {
                    // Issues deserializing...
                    _aux.RenderErrorInternalError(context, "Error reading XML resource stream: not a valid CodeXRC file array XML stream. Please check your XML generation algorithm.");
                    return;
                }
            }
            catch
            {
                // Issues deserializing...
                _aux.RenderErrorInternalError(context, "Error reading XML resource stream: could not deserialize the object. Please check your XML generation algorithm.");
                return;
            }

            try
            {
                // Create or update any files in this folder
                if (files.Count() > 0)
                {
                    // Pull a file ID...if -1, we're creating new
                    // files. If not, we're updating one.
                    if (files[0].FileID >= 0)
                    {
                        // Updating...
                        createFiles = false;
                    }

                    foreach (FileDTO file in files)
                    {
                        // Create or update this file
                        _dal.CreateOrUpdateFile(context, ownerID, projectName, file.FolderID, file);

                        // To reduce the size of the returned XML, we'll remove
                        // the file contents.
                        file.Contents = String.Empty;
                    }
                }
            }
            catch (CodeXRCException)
            {
                // Return error already formatted and issued...
            }
            catch (Exception ex)
            {
                // Issues creating new file...
                _aux.RenderErrorInternalError(context, String.Format("Error recording new file resource: {0}.", ex.Message));
            }

            // Assign proper return code and tweak headers...
            if (createFiles)
            {
                // Return created/updated project to client
                serializer.WriteObject(context.HttpContext.Response.OutputStream, files);

                // Indicate it has been created...note in this case we'll return
                // the folder in which the files are located.
                context.HttpContext.Response.Headers.Add("Location", CodeXRCLinkBuilder.BuildFolderLink(context, projectName, folderName));

                // Assign created return code
                _aux.RenderSuccessCreated(context);
            }
            else
            {
                // Assign OK return code
                _aux.RenderSuccessOK(context);
            }
        }

        private void CreateFile(ControllerContext context, Guid ownerID, string projectName, string folderName, string fileName)
        {
            // Deserialize the XML
            DataContractSerializer serializer = new DataContractSerializer(typeof(FileDTO));
            FileDTO file = null;
            bool createFile = true;
            try
            {
                // Deserialize
                file = serializer.ReadObject(context.HttpContext.Request.InputStream) as FileDTO;

                // Check to see that we have an object
                if (file == null)
                {
                    // Issues deserializing...
                    _aux.RenderErrorInternalError(context, "Error reading XML resource stream: not a valid CodeXRC file XML stream. Please check your XML generation algorithm.");
                    return;
                }

                // Pull the file ID...if -1, we're creating a new
                // file. If not, we're updating it.
                if (file.FileID >= 0)
                {
                    // Updating...
                    createFile = false;
                }
            }
            catch
            {
                // Issues deserializing...
                _aux.RenderErrorInternalError(context, "Error reading XML resource stream: could not deserialize the object. Please check your XML generation algorithm.");
                return;
            }

            try
            {
                // Create or update this file
                _dal.CreateOrUpdateFile(context, ownerID, projectName, file.FolderID, file);

                // To reduce the size of the returned XML, we'll remove
                // the file contents.
                file.Contents = String.Empty;
            }
            catch (CodeXRCException)
            {
                // Return error already formatted and issued...
            }
            catch (Exception ex)
            {
                // Issues creating new file...
                _aux.RenderErrorInternalError(context, String.Format("Error recording new file resource: {0}.", ex.Message));
            }

            // Assign proper return code and tweak headers...
            if (createFile)
            {
                // Return created/updated project to client
                serializer.WriteObject(context.HttpContext.Response.OutputStream, file);

                // Indicate it has been created
                string extension = file.DisplayName.LastIndexOf('.') != -1 ? file.DisplayName.Substring(fileName.LastIndexOf('.') + 1) : String.Empty;
                string name = file.DisplayName.LastIndexOf('.') != -1 ? file.DisplayName.Substring(0, file.DisplayName.Length - file.DisplayName.LastIndexOf('.')) : fileName;
                context.HttpContext.Response.Headers.Add("Location", CodeXRCLinkBuilder.BuildFileLink(context, projectName, folderName, extension, name));

                // Assign created return code
                _aux.RenderSuccessCreated(context);
            }
            else
            {
                // Assign OK return code
                _aux.RenderSuccessOK(context);
            }
        }
    }
}
